import sys
import unittest
from util import run_stron


ENCODING = 'utf-8'


STRON1 = '''\
<?xml version="1.0" encoding="UTF-8"?>
<sch:schema xmlns:sch="http://purl.oclc.org/dsdl/schematron" version="ISO">
 <sch:title>Example Schematron Schema</sch:title>
 <sch:pattern>
   <sch:rule context="dog">
    <sch:assert test="count(ear) = 2"
    >A 'dog' element should contain two 'ear' elements.</sch:assert>
    <sch:report test="bone"
    >This dog has a bone.</sch:report>
   </sch:rule>
  </sch:pattern>
</sch:schema>
'''

STRON2 = '''\
<?xml version="1.0" encoding="UTF-8"?>
<sch:schema xmlns:sch="http://purl.oclc.org/dsdl/schematron" version="ISO">
 <sch:title>Example Schematron Schema</sch:title>
 <sch:pattern>
   <sch:rule context="dog">
    <sch:assert test="count(ear) = 2"
    >A '<sch:name/>' element should contain two 'ear' elements. Test value: <sch:value-of select="10*10"/></sch:assert>
    <sch:report test="bone"
    >This dog has a bone.</sch:report>
   </sch:rule>
  </sch:pattern>
</sch:schema>
'''

STRON3 = '''\
<?xml version="1.0" encoding="UTF-8"?>
<sch:schema xmlns:sch="http://purl.oclc.org/dsdl/schematron" version="ISO">
 <sch:title>Example Schematron Schema</sch:title>
 <sch:pattern>
  <sch:rule context="dog" >
   <sch:assert test="nose | @exceptional='true'" diagnostics="d1 d2"
   >A dog should have a nose.</sch:assert>
  </sch:rule>
 </sch:pattern>

 <sch:diagnostics>
 <sch:diagnostic id="d1"
 >Your dog <sch:value-of select="@petname" /> has no nose. 
 How does he smell? Terrible. Give him a nose element,
 putting it after the <sch:name path="child::*[2]"/> element.
 </sch:diagnostic>
 <sch:diagnostic id="d2"
 >Animals such as <sch:name/> usually come with a full complement
 of legs, ears and noses, etc. However, exceptions are possible.
 If your dog is exceptional, provide an attribute
 <sch:emph>exceptional='true'</sch:emph>
 </sch:diagnostic>
</sch:diagnostics>
</sch:schema>
'''

STRON4 = '''\
<?xml version="1.0" encoding="UTF-8"?>
<sch:schema xmlns:sch="http://purl.oclc.org/dsdl/schematron" version="ISO">
 <sch:title>Example with problem patterns</sch:title>
 <sch:pattern>
 </sch:pattern>
 <sch:pattern>
   <sch:rule>
    <sch:assert test="count(ear) = 2"
    >A 'dog' element should contain two 'ear' elements.</sch:assert>
   </sch:rule>
 </sch:pattern>
</sch:schema>
'''

STRON5 = '''\
<?xml version="1.0" encoding="UTF-8"?>
<sch:schema xmlns:sch="http://purl.oclc.org/dsdl/schematron" version="ISO" %s>
 <sch:title>Example Schematron Schema</sch:title>
 <sch:phase id='PH1'>
  <sch:active pattern='PAT1'/>
 </sch:phase>
 <sch:phase id='PH2'>
  <sch:active pattern='PAT2'/>
 </sch:phase>
 <sch:phase id='PH3'>
  <sch:active pattern='PAT1'/>
  <sch:active pattern='PAT2'/>
 </sch:phase>
 <!-- The two rules int he two patterns are redundant.  This is just for testing -->
 <sch:pattern id='PAT1'>
   <sch:rule context="dog">
    <sch:assert test="count(ear) = 2"
    >A 'dog' element should contain two 'ear' elements.</sch:assert>
   </sch:rule>
  </sch:pattern>
  <sch:pattern id='PAT2'>
   <sch:rule context="dog">
    <sch:assert test="name(*[2]) = 'ear'"
    >The second child of a '<sch:name/>' element should be an 'ear' element.</sch:assert>
   </sch:rule>
  </sch:pattern>
</sch:schema>
'''

STRON6 = '''\
<?xml version="1.0" encoding="UTF-8"?>
<sch:schema xmlns:sch="http://purl.oclc.org/dsdl/schematron" version="ISO" %s>
 <sch:title>Example Schematron Schema</sch:title>
 <sch:pattern>
   <sch:rule context="dog">
    <sch:assert test="generate-id(ear) != ''">Should never fire.</sch:assert>
   </sch:rule>
  </sch:pattern>
</sch:schema>
'''

STRON7 = '''\
<?xml version="1.0" encoding="UTF-8"?>
<sch:schema xmlns:sch="http://purl.oclc.org/dsdl/schematron"
            version="ISO" %s>
 <sch:title>Example Schematron Schema</sch:title>
 <sch:ns uri="http://exslt.org/regular-expressions" prefix="regex"/>
 <sch:pattern>
   <sch:rule context="dog">
    <sch:assert test="regex:test(name(*), '.*s')">The first child element's name should end with 's'.</sch:assert>
    <sch:assert test="count(regex:match(name(*), 'e')) = 1">The first child element's name should contain one 'e'.</sch:assert>
   </sch:rule>
  </sch:pattern>
</sch:schema>
'''

STRON8 = '''\
<?xml version="1.0" encoding="UTF-8"?>
<sch:schema xmlns:sch="http://purl.oclc.org/dsdl/schematron" version="ISO" %s>
 <sch:title>Example Schematron Schema</sch:title>
 <sch:pattern>
   <sch:rule context="a">
    <sch:assert test="count(b[normalize-space() = current()/b[position() = 1]]) != 2">Should never fire.</sch:assert>
   </sch:rule>
  </sch:pattern>
</sch:schema>
'''

STRON_EVDV_060812 = '''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron" queryBinding="exslt">
    <ns prefix="regexp" uri="http://exslt.org/regular-expressions"/>
    <pattern>
        <rule context="*">
          <let name="matches" value="regexp:match(normalize-space(.), '\.')"/>
          <report test="$matches">Found a dot in <name/></report>
        </rule>
    </pattern>
</schema>
'''

STRON_EVDV_060914 = '''\
<schema xmlns="http://purl.oclc.org/dsdl/schematron" queryBinding="exslt">
  <ns prefix="regexp" uri="http://exslt.org/regular-expressions"/>
  <pattern fpi="test-doc">
    <rule context="/">
      <assert test="document('dummy.xml')">Unable to load document</assert>
    </rule>
  </pattern>
</schema>
'''

STRONX = '''\
<?xml version="1.0" encoding="UTF-8"?>
<sch:schema xmlns:sch="http://purl.oclc.org/dsdl/schematron"
            xmlns:regex="http://exslt.org/regular-expressions"
            version="ISO" %s>
 <sch:title>Example Schematron Schema</sch:title>
 <sch:pattern>
   <sch:rule context="dog">
    <sch:assert test="regex:match(name(*), 'e.*')">The first child element's name should start with 'e'.</sch:assert>
   </sch:rule>
  </sch:pattern>
</sch:schema>
'''

OBSOLETE_STRON = '''\
<?xml version="1.0" encoding="UTF-8"?>
<sch:schema xmlns:sch="http://www.ascc.net/xml/schematron" version="ISO">
 <sch:title>Example Schematron Schema</sch:title>
 <sch:pattern>
   <sch:rule context="dog">
    <sch:assert test="count(ear) = 2"
    >A 'dog' element should contain two 'ear' elements.</sch:assert>
    <sch:report test="bone"
    >This dog has a bone.</sch:report>
   </sch:rule>
  </sch:pattern>
</sch:schema>
'''


INSTANCE1 = u'<dog><ear/><ear/></dog>'.encode(ENCODING)

INSTANCE2 = u'''\
<dog><ear/></dog>'''.encode(ENCODING)

INSTANCE3 = u'''\
<dog><ear/><bone/><ear/></dog>'''.encode(ENCODING)

INSTANCE4 = u'''\
<dog><ear/><bone/></dog>'''.encode(ENCODING)

INSTANCE5 = u'''\
<a><b>.</b><c></c><d> . </d></a>'''.encode(ENCODING)


def normalize_text(text):
    #normalize text, a bit slow for large files with much non-ws
    return reduce(lambda a, b: (a and a[-1].isspace() and b.isspace()) and a or a+b, text, '')


class TestBasics(unittest.TestCase):
    def testStron1(self):
        output = run_stron(STRON1, INSTANCE1)
        #print output
        #normalize text, a bit slow for large files with much non-ws
        output = normalize_text(output)
        #print output

        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Example Schematron Schema\nProcessing pattern: [unnamed]\n'
        self.assertEqual(output, expected)
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testStron2(self):
        output = run_stron(STRON1, INSTANCE3)
        #print output
        #normalize text, a bit slow for large files with much non-ws
        output = normalize_text(output)
        #print output

        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Example Schematron Schema\nProcessing pattern: [unnamed]\nReport:\nThis dog has a bone.\n'
        self.assertEqual(output, expected)
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testStron3(self):
        output = run_stron(STRON1, INSTANCE2)
        output = normalize_text(output)

        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Example Schematron Schema\nProcessing pattern: [unnamed]\nAssertion failure:\nA \'dog\' element should contain two \'ear\' elements.\n'
        self.assertEqual(output, expected)
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testStron4(self):
        output = run_stron(STRON3, INSTANCE1)
        #print output
        #normalize text, a bit slow for large files with much non-ws
        output = normalize_text(output)
        #print output

        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Example Schematron Schema\nProcessing pattern: [unnamed]\nAssertion failure:\nA dog should have a nose.\nDiagnostic message:\nYour dog has no nose. How does he smell? Terrible. Give him a nose element,\nputting it after the ear element.\nDiagnostic message:\nAnimals such as dog usually come with a full complement\nof legs, ears and noses, etc. However, exceptions are possible.\nIf your dog is exceptional, provide an attribute\n<emph>exceptional=\'true\'</emph>\n'
        self.assertEqual(output, expected)
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testStron5(self):
        output = run_stron(STRON3, INSTANCE2)
        #print output
        #normalize text, a bit slow for large files with much non-ws
        output = normalize_text(output)
        #print output

        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Example Schematron Schema\nProcessing pattern: [unnamed]\nAssertion failure:\nA dog should have a nose.\nDiagnostic message:\nYour dog has no nose. How does he smell? Terrible. Give him a nose element,\nputting it after the element.\nDiagnostic message:\nAnimals such as dog usually come with a full complement\nof legs, ears and noses, etc. However, exceptions are possible.\nIf your dog is exceptional, provide an attribute\n<emph>exceptional=\'true\'</emph>\n'
        self.assertEqual(output, expected)
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testStron6(self):
        output = run_stron(STRON3, INSTANCE3)
        #print output
        #normalize text, a bit slow for large files with much non-ws
        output = normalize_text(output)
        #print output

        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Example Schematron Schema\nProcessing pattern: [unnamed]\nAssertion failure:\nA dog should have a nose.\nDiagnostic message:\nYour dog has no nose. How does he smell? Terrible. Give him a nose element,\nputting it after the bone element.\nDiagnostic message:\nAnimals such as dog usually come with a full complement\nof legs, ears and noses, etc. However, exceptions are possible.\nIf your dog is exceptional, provide an attribute\n<emph>exceptional=\'true\'</emph>\n'
        self.assertEqual(output, expected)
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testStron7(self):
        output = run_stron(STRON1, INSTANCE4)
        #print output
        #normalize text, a bit slow for large files with much non-ws
        output = normalize_text(output)
        #print output

        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Example Schematron Schema\nProcessing pattern: [unnamed]\nAssertion failure:\nA \'dog\' element should contain two \'ear\' elements.\nReport:\nThis dog has a bone.\n'
        self.assertEqual(output, expected)
        return

    def testStron8(self):
        self.assertRaises(TypeError,
                          run_stron, STRON4, INSTANCE1)
        return

    def testStron9(self):
        #Default phase
        output = run_stron(STRON5%'', INSTANCE2)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Example Schematron Schema\nProcessing pattern: [unnamed]\nAssertion failure:\nA \'dog\' element should contain two \'ear\' elements.\nProcessing pattern: [unnamed]\nAssertion failure:\nThe second child of a \'dog\' element should be an \'ear\' element.\n'
        self.assertEqual(output, expected)

    def testStron10(self):
        output = run_stron(STRON5%'', INSTANCE2, phase='#ALL')
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Example Schematron Schema\nProcessing pattern: [unnamed]\nAssertion failure:\nA \'dog\' element should contain two \'ear\' elements.\nProcessing pattern: [unnamed]\nAssertion failure:\nThe second child of a \'dog\' element should be an \'ear\' element.\n'
        self.assertEqual(output, expected)

    def testStron11(self):
        output = run_stron(STRON5%'', INSTANCE2, phase='#DEFAULT')
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Example Schematron Schema\nProcessing pattern: [unnamed]\nAssertion failure:\nA \'dog\' element should contain two \'ear\' elements.\nProcessing pattern: [unnamed]\nAssertion failure:\nThe second child of a \'dog\' element should be an \'ear\' element.\n'
        self.assertEqual(output, expected)

    def testStron12(self):
        output = run_stron(STRON5%'', INSTANCE2, phase='PH1')
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Example Schematron Schema\nProcessing pattern: [unnamed]\nAssertion failure:\nA \'dog\' element should contain two \'ear\' elements.\n'
        self.assertEqual(output, expected)

    def testStron13(self):
        output = run_stron(STRON5%'', INSTANCE2, phase='PH2')
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Example Schematron Schema\nProcessing pattern: [unnamed]\nAssertion failure:\nThe second child of a \'dog\' element should be an \'ear\' element.\n'
        self.assertEqual(output, expected)
        return

    def testStron14(self):
        output = run_stron(STRON5%'', INSTANCE2, phase='PH3')
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Example Schematron Schema\nProcessing pattern: [unnamed]\nAssertion failure:\nA \'dog\' element should contain two \'ear\' elements.\nProcessing pattern: [unnamed]\nAssertion failure:\nThe second child of a \'dog\' element should be an \'ear\' element.\n'
        self.assertEqual(output, expected)

    def testStron15(self):
        output = run_stron(STRON5%' defaultPhase="PH2"', INSTANCE2)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Example Schematron Schema\nProcessing pattern: [unnamed]\nAssertion failure:\nThe second child of a \'dog\' element should be an \'ear\' element.\n'
        self.assertEqual(output, expected)

    def testStron16(self):
        output = run_stron(STRON6%' queryBinding="xslt"', INSTANCE1)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Example Schematron Schema\nProcessing pattern: [unnamed]\n'
        self.assertEqual(output, expected)

    def testStron17(self):
        from Ft.Xml.XPath import RuntimeException
        self.assertRaises(RuntimeException,
                          run_stron, STRON6%' queryBinding="xpath"', INSTANCE1)

    def testStron18(self):
        output = run_stron(STRON7%' queryBinding="exslt"', INSTANCE1)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Example Schematron Schema\nProcessing pattern: [unnamed]\nAssertion failure:\nThe first child element\'s name should end with \'s\'.\n'
        self.assertEqual(output, expected)

    def testStron19(self):
        output = run_stron(STRON8%' queryBinding="exslt"', INSTANCE5)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Example Schematron Schema\nProcessing pattern: [unnamed]\n'
        self.assertEqual(output, expected)

    def testStronVdv1(self):
        output = run_stron(STRON_EVDV_060812, INSTANCE5)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Processing pattern: [unnamed]\nReport:\nFound a dot in a\nReport:\nFound a dot in b\nReport:\nFound a dot in d\n'
        self.assertEqual(output, expected)

    def testStronVdv2(self):
        from Ft.Xml import InputSource
        from Ft.Lib import Uri
        isrc_factory = InputSource.DefaultFactory
        stron_isrc = isrc_factory.fromString(STRON_EVDV_060914, Uri.OsPathToUri(''))
        #from Ft.Xml import CreateInputSource
        output = run_stron(stron_isrc, INSTANCE5)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Processing pattern: [unnamed]\n'
        self.assertEqual(output, expected)



class TestOddCases(unittest.TestCase):
    def testObsoleteNamespace(self):
        self.assertRaises(RuntimeError,
                          run_stron, OBSOLETE_STRON, INSTANCE1)
        #import warnings
        #warnings.filterwarnings('error', '.*', DeprecationWarning)
        #self.assertRaises(DeprecationWarning,
        #                  run_stron, OBSOLETE_STRON, INSTANCE1)


if __name__ == '__main__':
    unittest.main()




__all__ = ["DeclHandler", "LexicalHandler", 'all_features', 'all_properties', 'feature_external_ges', 'feature_external_pes', 'feature_namespace_prefixes', 'feature_namespaces', 'feature_string_interning', 'feature_validation', 'property_declaration_handler', 'property_dom_node', 'property_encoding', 'property_interning_dict', 'property_lexical_handler', 'property_xml_string']


#Snipped from xml.sax.saxlib
"""
This module contains the core classes of version 2.0 of SAX for Python.
This file provides only default classes with absolutely minimum
functionality, from which drivers and applications can be subclassed.

Many of these classes are empty and are included only as documentation
of the interfaces.

$Id: pyxml_standins.py,v 1.2 2004/12/30 06:13:45 uogbuji Exp $
"""

# A number of interfaces used to live in saxlib, but are now in
# various other modules for Python 2 compatibility. If nobody uses
# them here any longer, the references can be removed

from xml.sax.handler import ErrorHandler, ContentHandler, DTDHandler, EntityResolver
from xml.sax.xmlreader import XMLReader, InputSource, Locator, IncrementalParser

#============================================================================
#
# HANDLER INTERFACES
#
#============================================================================


# ===== DECLHANDLER =====

class DeclHandler:
    """Optional SAX2 handler for DTD declaration events.

    Note that some DTD declarations are already reported through the
    DTDHandler interface. All events reported to this handler will
    occur between the startDTD and endDTD events of the
    LexicalHandler.

    To set the DeclHandler for an XMLReader, use the setProperty method
    with the identifier http://xml.org/sax/handlers/DeclHandler."""

    def attributeDecl(self, elem_name, attr_name, type, value_def, value):
        """Report an attribute type declaration.

        Only the first declaration will be reported. The type will be
        one of the strings "CDATA", "ID", "IDREF", "IDREFS",
        "NMTOKEN", "NMTOKENS", "ENTITY", "ENTITIES", or "NOTATION", or
        a list of names (in the case of enumerated definitions).

        elem_name is the element type name, attr_name the attribute
        type name, type a string representing the attribute type,
        value_def a string representing the default declaration
        ('#IMPLIED', '#REQUIRED', '#FIXED' or None). value is a string
        representing the attribute's default value, or None if there
        is none."""

    def elementDecl(self, elem_name, content_model):
        """Report an element type declaration.

        Only the first declaration will be reported.

        content_model is the string 'EMPTY', the string 'ANY' or the content
        model structure represented as tuple (separator, tokens, modifier)
        where separator is the separator in the token list (that is, '|' or
        ','), tokens is the list of tokens (element type names or tuples
        representing parentheses) and modifier is the quantity modifier
        ('*', '?' or '+')."""

    def internalEntityDecl(self, name, value):
        """Report an internal entity declaration.

        Only the first declaration of an entity will be reported.

        name is the name of the entity. If it is a parameter entity,
        the name will begin with '%'. value is the replacement text of
        the entity."""

    def externalEntityDecl(self, name, public_id, system_id):
        """Report a parsed entity declaration. (Unparsed entities are
        reported to the DTDHandler.)

        Only the first declaration for each entity will be reported.

        name is the name of the entity. If it is a parameter entity,
        the name will begin with '%'. public_id and system_id are the
        public and system identifiers of the entity. public_id will be
        None if none were declared."""



# ===== LEXICALHANDLER =====

class LexicalHandler:
    """Optional SAX2 handler for lexical events.

    This handler is used to obtain lexical information about an XML
    document, that is, information about how the document was encoded
    (as opposed to what it contains, which is reported to the
    ContentHandler), such as comments and CDATA marked section
    boundaries.

    To set the LexicalHandler of an XMLReader, use the setProperty
    method with the property identifier
    'http://xml.org/sax/handlers/LexicalHandler'. There is no
    guarantee that the XMLReader will support or recognize this
    property."""

    def comment(self, content):
        """Reports a comment anywhere in the document (including the
        DTD and outside the document element).

        content is a string that holds the contents of the comment."""

    def startDTD(self, name, public_id, system_id):
        """Report the start of the DTD declarations, if the document
        has an associated DTD.

        A startEntity event will be reported before declaration events
        from the external DTD subset are reported, and this can be
        used to infer from which subset DTD declarations derive.

        name is the name of the document element type, public_id the
        public identifier of the DTD (or None if none were supplied)
        and system_id the system identfier of the external subset (or
        None if none were supplied)."""

    def endDTD(self):
        "Signals the end of DTD declarations."

    def startEntity(self, name):
        """Report the beginning of an entity.

        The start and end of the document entity is not reported. The
        start and end of the external DTD subset is reported with the
        pseudo-name '[dtd]'.

        Skipped entities will be reported through the skippedEntity
        event of the ContentHandler rather than through this event.

        name is the name of the entity. If it is a parameter entity,
        the name will begin with '%'."""

    def endEntity(self, name):
        """Reports the end of an entity. name is the name of the
        entity, and follows the same conventions as for
        startEntity."""

    def startCDATA(self):
        """Reports the beginning of a CDATA marked section.

        The contents of the CDATA marked section will be reported
        through the characters event."""

    def endCDATA(self):
        "Reports the end of a CDATA marked section."

#
#From xml.sax.handler
#
#============================================================================
#
# CORE FEATURES
#
#============================================================================

feature_namespaces = "http://xml.org/sax/features/namespaces"
# true: Perform Namespace processing (default).
# false: Optionally do not perform Namespace processing
#        (implies namespace-prefixes).
# access: (parsing) read-only; (not parsing) read/write

feature_namespace_prefixes = "http://xml.org/sax/features/namespace-prefixes"
# true: Report the original prefixed names and attributes used for Namespace
#       declarations.
# false: Do not report attributes used for Namespace declarations, and
#        optionally do not report original prefixed names (default).
# access: (parsing) read-only; (not parsing) read/write

feature_string_interning = "http://xml.org/sax/features/string-interning"
# true: All element names, prefixes, attribute names, Namespace URIs, and
#       local names are interned using the built-in intern function.
# false: Names are not necessarily interned, although they may be (default).
# access: (parsing) read-only; (not parsing) read/write

feature_validation = "http://xml.org/sax/features/validation"
# true: Report all validation errors (implies external-general-entities and
#       external-parameter-entities).
# false: Do not report validation errors.
# access: (parsing) read-only; (not parsing) read/write

feature_external_ges = "http://xml.org/sax/features/external-general-entities"
# true: Include all external general (text) entities.
# false: Do not include external general entities.
# access: (parsing) read-only; (not parsing) read/write

feature_external_pes = "http://xml.org/sax/features/external-parameter-entities"# true: Include all external parameter entities, including the external
#       DTD subset.
# false: Do not include any external parameter entities, even the external
#        DTD subset.
# access: (parsing) read-only; (not parsing) read/write

all_features = [feature_namespaces,
                feature_namespace_prefixes,
                feature_string_interning,
                feature_validation,
                feature_external_ges,
                feature_external_pes]


#============================================================================
#
# CORE PROPERTIES
#
#============================================================================

property_lexical_handler = "http://xml.org/sax/properties/lexical-handler"
# data type: xml.sax.sax2lib.LexicalHandler
# description: An optional extension handler for lexical events like comments.
# access: read/write

property_declaration_handler = "http://xml.org/sax/properties/declaration-handler"
# data type: xml.sax.sax2lib.DeclHandler
# description: An optional extension handler for DTD-related events other
#              than notations and unparsed entities.
# access: read/write

property_dom_node = "http://xml.org/sax/properties/dom-node"
# data type: org.w3c.dom.Node
# description: When parsing, the current DOM node being visited if this is
#              a DOM iterator; when not parsing, the root DOM node for
#              iteration.
# access: (parsing) read-only; (not parsing) read/write

property_xml_string = "http://xml.org/sax/properties/xml-string"
# data type: String
# description: The literal string of characters that was the source for
#              the current event.
# access: read-only

property_encoding = "http://www.python.org/sax/properties/encoding"
# data type: String
# description: The name of the encoding to assume for input data.
# access: write: set the encoding, e.g. established by a higher-level
#                protocol. May change during parsing (e.g. after
#                processing a META tag)
#         read:  return the current encoding (possibly established through
#                auto-detection.
# initial value: UTF-8
#

property_interning_dict = "http://www.python.org/sax/properties/interning-dict"
# data type: Dictionary
# description: The dictionary used to intern common strings in the document
# access: write: Request that the parser uses a specific dictionary, to
#                allow interning across different documents
#         read:  return the current interning dictionary, or None
#

all_properties = [property_lexical_handler,
                  property_dom_node,
                  property_declaration_handler,
                  property_xml_string,
                  property_encoding,
                  property_interning_dict]





import unittest
from xml.dom import Node
from amara import domtools

#Tests examples given in the manual
class TestMJ030815(unittest.TestCase):
    XML = """\
<?xml version="1.0" encoding="iso-8859-1"?>
<labels>
  <label id='ep' added="2003-06-10">
    <name>Ezra Pound</name>
    <address>
      <street>45 Usura Place</street>
      <city>Hailey</city>
      <province>ID</province>
    </address>
  </label>
  <label id='tse' added="2003-06-20">
    <name>Thomas Eliot</name>
    <address>
      <street>3 Prufrock Lane</street>
      <city>Stamford</city>
      <province>CT</province>
    </address>
  </label>
  <label id="lh" added="2004-11-01">
    <name>Langston Hughes</name>
    <address>
      <street>10 Bridge Tunnel</street>
      <city>Harlem</city>
      <province>NY</province>
    </address>
  </label>
  <label id="co" added="2004-11-15">
    <name>Christopher Okigbo</name>
    <address>
      <street>7 Heaven's Gate</street>
      <city>Idoto</city>
      <province>Anambra</province>
    </address>
  </label>
</labels>
"""
    
    def testPushFromString(self):
        results = []
        for docfrag in domtools.pushdom(self.XML, u'/labels/label'):
            label = docfrag.firstChild
            name = label.xpath('string(name)')
            city = label.xpath('string(address/city)')
            if name.lower().find('eliot') != -1:
                results.append(city)
        self.assertEqual(results, [u'Stamford'])
        return


#Tests interesting patterns of XML documents with pushbind

LABELS = """\
<?xml version="1.0" encoding="iso-8859-1"?>
<labels>
  <label id="tse" added="2003-06-20">
    <name>Thomas Eliot</name>
    <address>
      <street>3 Prufrock Lane</street>
      <city>Stamford</city>
      <state>CT</state>
    </address>
    <quote>
      <emph>Midwinter Spring</emph> is its own season&#8230;
    </quote>
  </label>
  <label id="ep" added="2003-06-10">
    <name>Ezra Pound</name>
    <address>
      <street>45 Usura Place</street>
      <city>Hailey</city>
      <state>ID</state>
    </address>
    <quote>
      What thou lovest well remains, the rest is dross&#8230;
    </quote>
  </label>
  <!-- Throw in 10,000 more records just like this -->
  <label id="lh" added="2004-11-01">
    <name>Langston Hughes</name>
    <address>
      <street>10 Bridge Tunnel</street>
      <city>Harlem</city>
      <state>NY</state>
    </address>
  </label>
</labels>
"""

ROOT_GI_IN_CHILD = """\
<A>
  <A>One</A>
  <B>Two</B>
</A>
"""

MIXED_TOPLEVEL_CHILDREN_1 = """\
<A>
  <B id="1">One</B>
  <C id="2">Two</C>
  <B id="3">Three</B>
  <C id="3">Four</C>
</A>
"""

MIXED_TOPLEVEL_CHILDREN_2 = """\
<A>
  <B id="1">One</B>
  <C id="2">Two</C>
  <D id="3">Three</D>
  <B id="4">Four</B>
  <C id="5">Five</C>
  <D id="6">Six</D>
</A>
"""

MIXED_TOPLEVEL_CHILDREN_AND_GRANDKIDS = """\
<A>
  <B id="1">One</B>
  <C id="2">
    <B id="2.1">Two</B>
  </C>
  <B id="3">
    <C id="3.1">Three</C>
  </B>
  <C id="4">Three</C>
</A>
"""

ROOT_GI_IN_CHILD_GRANDCHILD = """\
<A>
  <A>
    <A>A One</A>
    <B>A Two</B>
  </A>
  <B>
    <A>B One</A>
    <B>B Two</B>
  </B>
</A>
"""

QNAME_CLASH_1 = """\
<a xmlns='http://example.com/ns1'>
  <a>One</a>
  <a xmlns='http://example.com/ns2'>Two</a>
  <b>Three</b>
</a>
"""


class TestRealWorld(unittest.TestCase):
    def testXmlcomArticle(self):
        names = []
        cities = []
        for docfrag in domtools.pushdom(LABELS, u'labels/label'):
            label = docfrag.firstChild
            name = label.xpath('string(name)')
            names.append(name)
            city = label.xpath('string(address/city)')
            cities.append(city)
        self.assertEqual(names, [u"Thomas Eliot", u"Ezra Pound", u"Langston Hughes"])
        self.assertEqual(cities, [u"Stamford", u"Hailey", u"Harlem"])
        return

class XTestBasicPushdom(unittest.TestCase):
    #def setUp(self):
    #    return

    def testToplevelKidsAndGrandkids1(self):
        subtrees = domtools.pushdom(MIXED_TOPLEVEL_CHILDREN_AND_GRANDKIDS, u"/A/*")
        l1 = []
        l2 = []
        for st in subtrees:
            e = st.firstChild
            self.assertEqual(e.parentNode.nodeType, Node.DOCUMENT_FRAGMENT_NODE)
            l1.append(e.nodeName)
            l2.append(e.getAttributeNS(None, 'id'))
        self.assertEqual(l1, [u"B", u"C", u"B", u"C"])
        self.assertEqual(l2, [u"1", u"2", u"3", u"4"])
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testToplevelKidsAndGrandkids2(self):
        subtrees = domtools.pushdom(MIXED_TOPLEVEL_CHILDREN_AND_GRANDKIDS, u"B")
        l1 = []
        for st in subtrees:
            e = st.firstChild
            self.assertEqual(e.parentNode.nodeType, Node.DOCUMENT_FRAGMENT_NODE)
            self.assertEqual(e.nodeName, u"B")
            l1.append(e.getAttributeNS(None, 'id'))
        self.assertEqual(l1, [u"1", u"2.1", u"3"])
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testQnameClash(self):
        PREFIXES = {u'ns1': u'http://example.com/ns1',
                    u'ns2': u'http://example.com/ns2'}
        subtrees = domtools.pushdom(QNAME_CLASH_1, u"/*/*")
        elem = subtrees.next().firstChild
        self.assertEqual(elem.namespaceURI, u'http://example.com/ns1')
        elem = subtrees.next().firstChild
        self.assertEqual(elem.namespaceURI, u'http://example.com/ns2')
        elem = subtrees.next().firstChild
        self.assertEqual(elem.namespaceURI, u'http://example.com/ns1')

        subtrees = domtools.pushdom(QNAME_CLASH_1, u"/ns1:a/*",
                                      prefixes=PREFIXES)
        elem = subtrees.next().firstChild
        self.assertEqual(elem.namespaceURI, u'http://example.com/ns1')
        elem = subtrees.next().firstChild
        self.assertEqual(elem.namespaceURI, u'http://example.com/ns2')
        elem = subtrees.next().firstChild
        self.assertEqual(elem.namespaceURI, u'http://example.com/ns1')

        subtrees = domtools.pushdom(QNAME_CLASH_1, u"/ns1:a",
                                      prefixes=PREFIXES)
        elem = subtrees.next().firstChild
        self.assertEqual(elem.namespaceURI, u'http://example.com/ns1')
        self.assertEqual(len(elem.firstChild.nextSibling.childNodes), 1)
        return


if __name__ == '__main__':
    unittest.main()



#Basic tests from stron 1.5 spec ( http://xml.ascc.net/resource/schematron/Schematron2000.html )
import sys
import unittest
from util import run_stron

ENCODING = 'utf-8'


STRON1 = '''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron">
  <title>Example of let</title>
  <pattern name="Nor too many nor too few dollars">
    <rule context="money">
      <!-- Remove dollar signs and US convention comma separators -->
      <let name="amount" value="number(translate(@amount, '$,', ''))"/>
      <assert test="$amount >= 10000 and $amount &lt;= 1000000">
        Amount should range from ten thousand to  one million dollars
      </assert>
    </rule>
  </pattern>
</schema>
'''

INSTANCE1 = u'<money amount="$300,000.00"/>'.encode(ENCODING)

INSTANCE2 = u'<money amount="500,000"/>'.encode(ENCODING)

INSTANCE3 = u'<money amount="10000.00"/>'.encode(ENCODING)

INSTANCE4 = u'<money amount="$1,000.00"/>'.encode(ENCODING)

INSTANCE5 = u'<money amount="$3,000,000.00"/>'.encode(ENCODING)


STRON2 = '''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron">
  <title>Table abstract patterns</title>
  <pattern abstract="true" id="table">
    <rule context="$table">
      <assert test="$row">A table has at least one row</assert>
    </rule>
    <rule context="$row">
      <assert test="$cell">A table row has at least one cell</assert>
    </rule>
  </pattern>

  <pattern name="xhtml-basic-table" is-a="table">
    <param formal="table" actual="table" />
    <param formal="row"  actual="tr" />
    <param formal="cell"  actual="td" />
  </pattern>

  <pattern name="cals-table" is-a="table">
    <param formal="table" actual="ctable" />
    <param formal="row"  actual="tbody/row" />
    <param formal="cell"  actual="entry" />
  </pattern>

  <pattern name="cals-table-extra">
    <rule context="ctable">
      <assert test="tbody">A table has a tbody</assert>
    </rule>
  </pattern>

</schema>
'''

STRON3 = STRON2

STRON_INVALID = '''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron">
  <title>Table abstract patterns</title>
  <pattern abstract="true" id="table">
    <rule context="$table">
      <assert test="$row">A table has at least one row</assert>
    </rule>
    <rule context="$row">
      <assert test="$cell">A table row has at least one cell</assert>
    </rule>
  </pattern>

  <pattern name="xhtml-basic-table" is-a="table">
    <param formal="table" actual="table" />
    <param formal="row"  actual="tr" />
    <param formal="cell"  actual="td" />
  </pattern>

  <pattern name="cals-table" is-a="table">
    <param formal="table" actual="ctable" />
    <param formal="row"  actual="tbody/row" />
    <param formal="cell"  actual="entry" />
    <!-- re:
( (attribute abstract { "true" }, title?, (p*, let*, rule*))
| (attribute abstract { "false" }?, title?, (p*, let*, rule*))
| (attribute abstract { "false" }?, attribute is-a { xsd:IDREF }, title?, (p*,
param*)
)
    -->
    <rule context="ctable">
      <assert test="tbody">A table has a tbody</assert>
    </rule>
  </pattern>

</schema>
'''

STRON4 = '''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron">
  <title>Table abstract patterns</title>
  <pattern abstract="true" id="table">
    <rule context="$table">
      <assert test="$row">A table has at least one row</assert>
    </rule>
    <rule context="$row">
      <assert test="$cell">A table row has at least one cell</assert>
    </rule>
    <rule context="$cell">
      <assert test="ancestor::$row">
        Cells can only appear in rows, not '<name path="parent::*"/>'
      </assert>
    </rule>     
  </pattern>

  <pattern name="xhtml-basic-table" is-a="table">
    <param formal="table" actual="table" />
    <param formal="row"  actual="tr" />
    <param formal="cell"  actual="td" />
  </pattern>

  <pattern name="cals-table" is-a="table">
    <param formal="table" actual="ctable" />
    <param formal="row"  actual="tbody/row" />
    <param formal="cell"  actual="entry" />
  </pattern>

  <pattern name="cals-table-extra">
    <rule context="ctable">
      <assert test="tbody">A table has a tbody</assert>
    </rule>
  </pattern>

</schema>
'''

INSTANCE10 = u'''\
<table>
  <tr>
    <td>hello world</td>
  </tr>
</table>
'''.encode(ENCODING)

INSTANCE11 = u'''\
<ctable>
  <tbody>
    <row>
      <entry>hello world</entry>
    </row>
  </tbody>
</ctable>
'''.encode(ENCODING)

INSTANCE12 = u'''\
<table>
  <tr>
  </tr>
</table>
'''.encode(ENCODING)

INSTANCE13 = u'''\
<ctable>
  <tbody>
    <row>
    </row>
  </tbody>
</ctable>
'''.encode(ENCODING)

INSTANCE14 = u'''\
<table>
  <td>
    <tr>
    </tr>
  </td>
</table>
'''.encode(ENCODING)

INSTANCE15 = u'''\
<ctable>
  <tbody>
    <entry>
      <row>
      </row>
    </entry>
  </tbody>
</ctable>
'''.encode(ENCODING)

INSTANCE16 = u'''\
<ctable>
    <row>
      <entry>hello world</entry>
    </row>
</ctable>
'''.encode(ENCODING)

#INSTANCE1 = u'<money amount="$300,000.00"/>'.encode(ENCODING)


def normalize_text(text):
    #normalize text, a bit slow for large files with much non-ws
    return reduce(lambda a, b: (a and a[-1].isspace() and b.isspace()) and a or a+b, text, '')


class TestBasics(unittest.TestCase):
    def setUp(self):
        #self.isrc_factory = InputSource.DefaultFactory
        return

    def test1_1(self):
        output = run_stron(STRON1, INSTANCE1)
        #print output
        #normalize text, a bit slow for large files with much non-ws
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Example of let\nProcessing pattern: Nor too many nor too few dollars\n'
        self.assertEqual(output, expected)
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def test1_2(self):
        output = run_stron(STRON1, INSTANCE2)
        #print output
        #normalize text, a bit slow for large files with much non-ws
        output = normalize_text(output)
        #print output

        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Example of let\nProcessing pattern: Nor too many nor too few dollars\n'
        self.assertEqual(output, expected)
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def test1_3(self):
        output = run_stron(STRON1, INSTANCE3)
        #print output
        #normalize text, a bit slow for large files with much non-ws
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Example of let\nProcessing pattern: Nor too many nor too few dollars\n'
        self.assertEqual(output, expected)
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def test1_4(self):
        output = run_stron(STRON1, INSTANCE4)
        #print output
        #normalize text, a bit slow for large files with much non-ws
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Example of let\nProcessing pattern: Nor too many nor too few dollars\nAssertion failure:\nAmount should range from ten thousand to one million dollars\n'
        self.assertEqual(output, expected)
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def test1_5(self):
        output = run_stron(STRON1, INSTANCE5)
        #print output
        #normalize text, a bit slow for large files with much non-ws
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Example of let\nProcessing pattern: Nor too many nor too few dollars\nAssertion failure:\nAmount should range from ten thousand to one million dollars\n'
        self.assertEqual(output, expected)
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def test2_10(self):
        output = run_stron(STRON2, INSTANCE10)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Table abstract patterns\nProcessing pattern: xhtml-basic-table\nProcessing pattern: cals-table\nProcessing pattern: cals-table-extra\n'
        self.assertEqual(output, expected)
        return

    def test2_11(self):
        output = run_stron(STRON2, INSTANCE11)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Table abstract patterns\nProcessing pattern: xhtml-basic-table\nProcessing pattern: cals-table\nProcessing pattern: cals-table-extra\n'
        self.assertEqual(output, expected)
        return

    def test2_12(self):
        output = run_stron(STRON2, INSTANCE12)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Table abstract patterns\nProcessing pattern: xhtml-basic-table\nAssertion failure:\nA table row has at least one cell\nProcessing pattern: cals-table\nProcessing pattern: cals-table-extra\n'
        self.assertEqual(output, expected)
        return

    def test2_13(self):
        output = run_stron(STRON2, INSTANCE13)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Table abstract patterns\nProcessing pattern: xhtml-basic-table\nProcessing pattern: cals-table\nAssertion failure:\nA table row has at least one cell\nProcessing pattern: cals-table-extra\n'
        self.assertEqual(output, expected)
        return

    def test2_14(self):
        output = run_stron(STRON2, INSTANCE14)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Table abstract patterns\nProcessing pattern: xhtml-basic-table\nAssertion failure:\nA table has at least one row\nAssertion failure:\nA table row has at least one cell\nProcessing pattern: cals-table\nProcessing pattern: cals-table-extra\n'
        self.assertEqual(output, expected)
        return

    def test2_15(self):
        output = run_stron(STRON2, INSTANCE15)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Table abstract patterns\nProcessing pattern: xhtml-basic-table\nProcessing pattern: cals-table\nAssertion failure:\nA table has at least one row\nProcessing pattern: cals-table-extra\n'
        self.assertEqual(output, expected)
        return

    def test2_16(self):
        output = run_stron(STRON2, INSTANCE16)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Table abstract patterns\nProcessing pattern: xhtml-basic-table\nProcessing pattern: cals-table\nAssertion failure:\nA table has at least one row\nProcessing pattern: cals-table-extra\nAssertion failure:\nA table has a tbody\n'
        self.assertEqual(output, expected)
        return

    def test3_10(self):
        output = run_stron(STRON3, INSTANCE10)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Table abstract patterns\nProcessing pattern: xhtml-basic-table\nProcessing pattern: cals-table\nProcessing pattern: cals-table-extra\n'
        self.assertEqual(output, expected)
        return

    def test3_11(self):
        output = run_stron(STRON3, INSTANCE11)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Table abstract patterns\nProcessing pattern: xhtml-basic-table\nProcessing pattern: cals-table\nProcessing pattern: cals-table-extra\n'
        self.assertEqual(output, expected)
        return

    def test3_12(self):
        output = run_stron(STRON3, INSTANCE12)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Table abstract patterns\nProcessing pattern: xhtml-basic-table\nAssertion failure:\nA table row has at least one cell\nProcessing pattern: cals-table\nProcessing pattern: cals-table-extra\n'
        self.assertEqual(output, expected)
        return

    def test3_13(self):
        output = run_stron(STRON3, INSTANCE13)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Table abstract patterns\nProcessing pattern: xhtml-basic-table\nProcessing pattern: cals-table\nAssertion failure:\nA table row has at least one cell\nProcessing pattern: cals-table-extra\n'
        self.assertEqual(output, expected)
        return

    def test3_14(self):
        output = run_stron(STRON3, INSTANCE14)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Table abstract patterns\nProcessing pattern: xhtml-basic-table\nAssertion failure:\nA table has at least one row\nAssertion failure:\nA table row has at least one cell\nProcessing pattern: cals-table\nProcessing pattern: cals-table-extra\n'
        self.assertEqual(output, expected)
        return

    def test3_15(self):
        output = run_stron(STRON3, INSTANCE15)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Table abstract patterns\nProcessing pattern: xhtml-basic-table\nProcessing pattern: cals-table\nAssertion failure:\nA table has at least one row\nProcessing pattern: cals-table-extra\n'
        self.assertEqual(output, expected)
        return

    def test3_16(self):
        output = run_stron(STRON3, INSTANCE16)
        output = normalize_text(output)
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Table abstract patterns\nProcessing pattern: xhtml-basic-table\nProcessing pattern: cals-table\nAssertion failure:\nA table has at least one row\nProcessing pattern: cals-table-extra\nAssertion failure:\nA table has a tbody\n'
        self.assertEqual(output, expected)
        return


if __name__ == '__main__':
    unittest.main()




from xml.dom import Node
from xml.dom import EMPTY_NAMESPACE as NULL_NAMESPACE
from xml.dom import EMPTY_PREFIX as NULL_PREFIX

from Ft.Xml.XPath.Context import Context
#from Ft.Xml.Xslt.XsltContext import XsltContext
from Ft.Xml.XPath import Evaluate
from Ft.Xml import SplitQName
from Ft.Xml import XML_NAMESPACE as XML_NS

#FIXME: Use 4Suite L10N
def _(t): return t


class dummy_node_wrapper(object):
    @property
    def docIndex(self):
        return 0

    #docIndex = property(_docIndex)

    def _xpathAttributes(self):
        return []

    xpathAttributes = property(_xpathAttributes)

    def _childNodes(self):
        return []

    childNodes = property(_childNodes)

    def __cmp__(self, other):
        return cmp(self.docIndex, other.docIndex)


class xpath_attr_wrapper(dummy_node_wrapper):
    nodeType = Node.ATTRIBUTE_NODE
    def __init__(self, qname, ns, value, parent):
        self.namespaceURI = ns
        self.prefix, self.localName = SplitQName(qname)
        self.value = value
        self.nodeValue = value
        self.name = qname
        self.nodeName = qname
        self.rootNode = parent._rootNode()
        #self.docIndex = id(value)
        return

    def __unicode__(self):
        return self.value


class text_wrapper(dummy_node_wrapper):
    nodeType = Node.TEXT_NODE
    def __init__(self, st, parent):
        self.data = st
        self.parentNode = parent
        #self.docIndex = id(st)
        return

    def __unicode__(self):
        return self.data

    def _rootNode(self):
        return self.parentNode._rootNode()

    rootNode = property(_rootNode)
    ownerDocument = rootNode

    @property
    def docIndex(self):
        #print "text_wrapper._docIndex"
        return 1

    #docIndex = property(_docIndex)


class xpath_wrapper_container_mixin(object):
    def xml_xpath(self, query):
        """
        Execute an XPath query with this object standing in for the
        context node.  The namespace mappings are taken from the root
        binding object.  There are no variable bindings.  Most XPath
        is supported.
        query - a unicode object expressing an XPath
        The return value depends on the XPath expression (expr)
        - If expr reults in an XPath string, the return value is a
        Python Unicode object
        - If expr reults in an XPath number, the return value is a
        Python float
        - If expr reults in an XPath boolean, the return value is a
        Python bool object
        - If expr reults in an XPath node set, the return value is a
        Python list (always a list, even if empty, or a node with just
        one entry)
        """
        prefixes = self.rootNode.xmlns_prefixes
        if u'xml' not in prefixes:
            prefixes.update({u'xml': XML_NS})
        ctx = Context(self, processorNss=prefixes)
        result = Evaluate(query, context=ctx)
        return result
    xpath_attr_wrapper.xml_xpath = xml_xpath

    def _namespaceURI(self):
        try:
            return self.xmlnsUri
        except AttributeError:
            return NULL_NAMESPACE

    namespaceURI = property(_namespaceURI)

    def _localName(self):
        try:
            return self.xmlnsLocalName
        except AttributeError:
            lname = self.nodeName
            if lname[0] == '#': lname = None
            return lname

    localName = property(_localName)

    def _prefix(self):
        try:
            return self.xmlnsPrefix
        except AttributeError:
            return NULL_PREFIX

    prefix = property(_prefix)

    def _parentNode(self):
        try:
            return self.xml_parent
        except AttributeError:
            return None

    parentNode = property(_parentNode)

    def _rootNode(self):
        if self.parentNode:
            return self.parentNode._rootNode()
        return self

    rootNode = property(_rootNode)
    ownerDocument = rootNode

    def _childNodes(self):
        children = []
        for node in self.xml_children:
            if hasattr(node, 'next_elem'):
                children.append(node)
            elif isinstance(node, unicode):
                children.append(text_wrapper(node, self))
            elif getattr(node, 'nodeType', None) == Node.PROCESSING_INSTRUCTION_NODE:
                children.append(node)
            elif getattr(node, 'nodeType', None) == Node.COMMENT_NODE:
                children.append(node)
            else:
                #Treat it as a text node bound to a Python type
                children.append(unicode(node))
        return children

    childNodes = property(_childNodes)

    def _xpathAttributes(self):
        try:
            attrs = self.xml_attributes
        except AttributeError:
            return []
        return [ xpath_attr_wrapper(qname, ns, unicode(getattr(self, attr)), self)
                 for attr, (qname, ns) in self.xml_attributes.items() ]

    xpathAttributes = property(_xpathAttributes)

    def _attributes(self):
        try:
            attrs = self.xml_attributes
        except AttributeError:
            return {}
        keys = [ (ns, SplitQName(qname)) 
                   for attr, (qname, ns) in self.xml_attributes.items() ]
        values = [ xpath_attr_wrapper(qname, ns, unicode(getattr(self, attr)), self)
                   for attr, (qname, ns) in self.xml_attributes.items() ]
        return dict(zip(keys, values))

    attributes = property(_attributes)

    def getAttributeNS(self, ns, local):
        try:
            attrs = self.xml_attributes
        except AttributeError:
            return {}
        keys = [ (ns, SplitQName(qname)[1]) 
                   for attr, (qname, ns) in self.xml_attributes.items() ]
        values = [ unicode(getattr(self, attr))
                   for attr, (qname, ns) in self.xml_attributes.items() ]
        attr_dict = dict(zip(keys, values))
        return attr_dict.get((ns, local), "")

    @property
    def docIndex(self):
        #print "_docIndex", self.nodeType
        ix = id(self)
        return ix > 0 and ix or -ix

    #docIndex = property(_docIndex)

    def __cmp__(self, other):
        if self is other: return 0
        try:
            return cmp(self.docIndex, other.docIndex)
        except AttributeError:
            return -cmp(other, unicode(self))


class BinderXpathException(Exception):
    pass

NO_ATTRIBUTE_SUPPORT = _('You have attempted to use an XPath expression that requires attribute support in the binding.  Try using the rule amara.binderytools.preserve_attribute_details.')




import os, sys
import tempfile
import cStringIO
from Ft.Xml import CreateInputSource
from amara import scimitar


def run_stron(stron_xml, candidate_xml, legacy_ns=False, phase=None):
    stron_isrc = CreateInputSource(stron_xml)
    candidate_isrc = CreateInputSource(candidate_xml)
    validator_fname = tempfile.mkstemp('.py')[1]
    validator_file = open(validator_fname, 'w')
    scimitar.run(stron_isrc, validator_file, legacy_ns, 1)
    validator_file.close()
    #import popen4
    #(cstdin, cstdout) = os.popen4('/usr/bin/python ' + validator_fname + ' -')
    #cstdin.write(INSTANCE1)
    #result = cstdout.read()
    TEST = 1
    env = {'TEST': 1}
    execfile(validator_fname, env)
    #print >> sys.stderr, validator_fname
    #print >> sys.stderr, env
    validate = env['validate']
    #xmlf = cStringIO.StringIO(candidate_xml)
    result = cStringIO.StringIO()
    validate(candidate_isrc, result, phase=phase)
    os.unlink(validator_fname)
    try:
        os.unlink(validator_fname + 'c')
    except OSError:
        pass
    return result.getvalue()





import unittest
from xml.dom import Node
from amara import domtools
from Ft.Xml import Parse

abs_path = domtools.abs_path
parse = Parse

#Below here should be identical to ../bindery/tools.py

class TestSB060223(unittest.TestCase):
    #Samuel L Bayer pointed out definciencies in the namespace handling
    #of domtools.abs_path
    #http://lists.fourthought.com/pipermail/4suite/2006-February/007757.html
    XML1 = '<foo xmlns:bar="http://bar.com"><baz/><bar:baz/></foo>'
    XML2 = '<foo xmlns="http://bax.com" xmlns:bar="http://bar.com"><baz/><bar:baz/></foo>'
    
    def testAbsPathWithNs(self):
        doc = parse(self.XML1)
        ap = abs_path(doc.documentElement.firstChild)
        self.assertEqual(ap, u'/foo[1]/baz[1]')
        ap = abs_path(doc.documentElement.firstChild.nextSibling)
        self.assertEqual(ap, u'/foo[1]/bar:baz[1]')
        return

    def testAbsPathWithDefaultNs(self):
        doc = parse(self.XML2)
        ap = abs_path(doc.documentElement.firstChild)
        self.assertEqual(ap, u'/amara.4suite.org.ns1:foo[1]/amara.4suite.org.ns1:baz[1]')
        ap = abs_path(doc.documentElement.firstChild.nextSibling)
        self.assertEqual(ap, u'/amara.4suite.org.ns1:foo[1]/bar:baz[1]')
        ap = abs_path(doc.documentElement.firstChild, {u'bax': u'http://bax.com'})
        self.assertEqual(ap, u'/bax:foo[1]/bax:baz[1]')
        ap = abs_path(doc.documentElement.firstChild.nextSibling, {u'bax': u'http://bax.com'})
        self.assertEqual(ap, u'/bax:foo[1]/bar:baz[1]')
        return


if __name__ == '__main__':
    unittest.main()




import itertools

import xml.sax
from xml.dom import EMPTY_NAMESPACE as NULL_NAMESPACE
from xml.dom import EMPTY_PREFIX as NULL_PREFIX
from xml.dom import XML_NAMESPACE
from xml.dom import Node
from Ft.Xml import Domlette, Sax, CreateInputSource

START_DOCUMENT               = 1
END_DOCUMENT                 = 2
START_ELEMENT                = 3
END_ELEMENT                  = 4
CHARACTER_DATA               = 10
COMMENT                      = 11
PI                           = 12

#
# namespace_mixin is a utility that helps manage namespace prefix mappings
#

class namespace_mixin:
    def __init__(self):
        self._ns_prefix = {XML_NAMESPACE: [u'xml'], NULL_NAMESPACE: [NULL_PREFIX]}
        self._prefix_ns = {u'xml': [XML_NAMESPACE], NULL_PREFIX: [NULL_NAMESPACE]}
        return
    
    def startPrefixMapping(self, prefix, uri):
        self._ns_prefix.setdefault(uri, []).append(prefix)
        self._prefix_ns.setdefault(prefix, []).append(uri)
        return

    def endPrefixMapping(self, prefix):
        uri = self._prefix_ns[prefix].pop()
        prefix = self._ns_prefix[uri].pop()
        #assert prefix == uri
        return

    def name_to_qname(self, name):
        #print self._ns_prefix
        #print self._prefix_ns
        uri, local = name
        prefix = self._ns_prefix[uri][-1]
        qname = ( prefix and ( prefix + u':' ) or '') + local
        return qname


def sniff_namespace(source):
    class report_nss:
        def __init__(self, parser):
            self.nss = {}
            self.parser = parser
            return

        def startPrefixMapping(self, prefix, uri):
            self.nss[prefix] = uri
            return
        
        def startElementNS(self, name, qname, attribs):
            self.parser.setProperty(Sax.PROPERTY_YIELD_RESULT, self.nss)
            return

    parser = Sax.CreateParser()

    parser.setFeature(xml.sax.handler.feature_external_pes, False)
    parser.setFeature(Sax.FEATURE_GENERATOR, True)
    handler = report_nss(parser)
    parser.setContentHandler(handler)
    elem_iter = parser.parse(CreateInputSource(source))
    nss = elem_iter.next()
    #for attribs in attribs_iterator:
    #     for name in attribs.keys(): print name
    return nss


class sax_writer:
    """
    A ContentHandler that serializes the result using a 4Suite writer
    """

    #def __init__(self, writer=XmlPrinter(sys.stdout, 'utf-8')):
    def __init__(self, writer, docfrag=False):
        """
        writer - object to handle output
        docfrag - True means assume the SAX input is a document fragment,
                  so ignore startDocument and endDocument events
        """
        self._docfrag = docfrag
        self._writer = writer
        try:
            self._writer.reset()
        except AttributeError:
            pass
        self._namespaces = {}
        return

    def startDocument(self):
        if not self._docfrag:
            self._writer.startDocument()
        return

    def endDocument(self):
        if not self._docfrag:
            self._writer.endDocument()
        return

    def startPrefixMapping(self, prefix, uri):
        self._namespaces[prefix] = uri
        return

    def startElementNS(self, (namespaceURI, localName), qualifiedName,
                       attributes):
        self._writer.startElement(qualifiedName, namespaceURI, 
                                   self._namespaces)
        self._namespaces = {}
        for (ns, local), value in attributes.items():
            qname = attributes.getQNameByName((ns, local))
            self._writer.attribute(qname, value, ns)
        return

    def endElementNS(self, (namespaceURI, localName), qualifiedName):
        self._writer.endElement(qualifiedName, namespaceURI)
        return

    def characters(self, data):
        self._writer.text(data)
        return


#
# Tenorsax framework: helps linerarize SAX logic
#

from xml import sax  #Python 2.3 or higher required
class tenorsax(namespace_mixin, sax.ContentHandler):
    def __init__(self, consumer):
        namespace_mixin.__init__(self)
        self.consumer = consumer
        self.dispatcher = consumer.top_dispatcher
        self.curr_gen = None
        return
    
    def startElementNS(self, name, qname, attributes):
        (ns, local) = name
        qname = self.name_to_qname(name)
        #print "Start element", (name, qname)
        self.consumer.event = (START_ELEMENT, ns, local)
        self.consumer.params = attributes
        self.curr_gen = tenorsax.event_loop_body(self.dispatcher, self.curr_gen, self.consumer.event)
        return
    
    def endElementNS(self, name, qname):
        (ns, local) = name
        qname = self.name_to_qname(name)
        #print "end element", (name, qname)
        self.consumer.event = (END_ELEMENT, ns, local)
        self.consumer.params = None
        self.curr_gen = tenorsax.event_loop_body(self.dispatcher, self.curr_gen, self.consumer.event)
        return
    
    def characters(self, text):
        #print "characters", text
        self.consumer.event = (CHARACTER_DATA,)
        self.consumer.params = text
        self.curr_gen = tenorsax.event_loop_body(self.dispatcher, self.curr_gen, self.consumer.event)
        return

    def event_loop_body(dispatcher, curr_gen, event):
        if curr_gen:
            curr_gen = tenorsax.execute_delegate(curr_gen)
        else:
            curr_gen = tenorsax.check_for_delegate(dispatcher, event)
        return curr_gen
    event_loop_body = staticmethod(event_loop_body)

    def execute_delegate(curr_gen):
        try:
            curr_gen.next()
        except StopIteration:
            curr_gen = None
        return curr_gen
    execute_delegate = staticmethod(execute_delegate)

    def check_for_delegate(dispatcher, event):
        if event[0] == START_ELEMENT:
            end_condition = (END_ELEMENT,) + event[1:]
        else:
            end_condition = None
        curr_gen = None
        delegate_generator = dispatcher.get(event)
        if delegate_generator:
            #Fire up the generator
            curr_gen = delegate_generator(end_condition)
            try:
                curr_gen.next()
            except StopIteration:
                print "immediate end"
                #Immediate generator termination
                curr_gen = None
        return curr_gen
    check_for_delegate = staticmethod(check_for_delegate)

#
#
#

from xml import sax
from xml.dom import XML_NAMESPACE, XMLNS_NAMESPACE
from xml.dom import EMPTY_NAMESPACE as NULL_NAMESPACE
from xml.dom import EMPTY_PREFIX as NULL_PREFIX

from Ft.Xml.Xslt import parser as XPatternParser

from Ft.Xml.Xslt.XPatterns import Patterns
from Ft.Xml.Xslt.XPatterns import Pattern
from Ft.Xml.Xslt.XPatterns import DocumentNodeTest
from Ft.Xml.XPath.ParsedNodeTest import LocalNameTest
from Ft.Xml.XPath.ParsedNodeTest import NamespaceTest
from Ft.Xml.XPath.ParsedNodeTest import QualifiedNameTest
from Ft.Xml.XPath.ParsedNodeTest import PrincipalTypeTest
from Ft.Xml.XPath import Context, Compile

DUMMY_DOCELEM = u'dummy'
START_STATE = 0
#A special state that means
#"immediately pop off the state stack and transit to the top state"
POP_STATE = 9999
TOP = -1
ANY = '?'
ATTRIB = '@'
#Used to figure out whether a wildcard event is user-specified,
#Or added internally
EXPLICIT, IMPLICIT = (True, False)


class xpattern_state_machine:
    """
    A simple state machine that interprets XPatterns
    A state is "live" when it represents the successful completion
    of an XPattern.
    """
    PARSER = XPatternParser.new()
    
    def __init__(self, repr_xp, xp, nss):
        self._state_table = {START_STATE: {}}
        self._live_states = {}
        self._attrib_tests = {}
        self._ignored_subtree_states = []
        self._push_states = []
        self._substate_depth = 0
        #States that should be hooked into for matching later XPatterns
        self._hook_states = {}
        newest_state = START_STATE
        last_state = START_STATE
        for subpat in xp.patterns:
            steps = subpat.steps[:]
            steps.reverse()
            for (step_count, (axis_type, node_test, ancestor)) in enumerate(steps):
                #Note: XSLT patterns only allow child or attribute axis
                handled = False
                attrib_test = None
                if axis_type == Node.ATTRIBUTE_NODE:
                    if (isinstance(node_test, LocalNameTest)
                        or isinstance(node_test, QualifiedNameTest)
                        or isinstance(node_test, NamespaceTest)
                        or isinstance(node_test, PrincipalTypeTest)
                        ):
                        attrib_test = node_test
                        start_event = (1, ATTRIB, None)
                        end_event = (0, ATTRIB, None)
                        handled = True
                elif isinstance(node_test, DocumentNodeTest):
                    start_event = (1, None, None)
                    end_event = (0, None, None)
                    handled = True
                elif isinstance(node_test, LocalNameTest):
                    if node_test.nodeType == Node.ELEMENT_NODE:
                        start_event = (1, None, node_test._name)
                        end_event = (0, None, node_test._name)
                        handled = True
                elif isinstance(node_test, QualifiedNameTest):
                    if node_test.nodeType == Node.ELEMENT_NODE:
                        ns = nss[node_test._prefix]
                        start_event = (1, ns, node_test._localName)
                        end_event = (0, ns, node_test._localName)
                        handled = True
                elif isinstance(node_test, PrincipalTypeTest):
                    if node_test.nodeType == Node.ELEMENT_NODE:
                        start_event = (1, ANY, EXPLICIT)
                        end_event = (0, ANY, EXPLICIT)
                        handled = True
                elif isinstance(node_test, NamespaceTest):
                    if node_test.nodeType == Node.ELEMENT_NODE:
                        ns = nss[node_test._prefix]
                        start_event = (1, ns, ANY)
                        end_event = (0, ns, ANY)
                        handled = True
                if not(handled):
                    import sys; print >> sys.stderr, "Pattern step not supported:", (axis_type, node_test, ancestor), "Node test class", node_test.__class__
                    continue

                #Say the full input is /u/v/w|/a/b/c and we're currently
                #Working the sub-pattern /a/b/c
                #top_state is the pattern 
                #last_state is the last

                last_state = newest_state
                newest_state += 1
                if not(step_count):
                    if attrib_test:
                    #    #Because we want to treat @X as */@X
                    #    start_event = (1, ANY, EXPLICIT)
                    #    end_event = (0, ANY, EXPLICIT)
                        attribute_test_state = newest_state
                    self._state_table[newest_state] = {end_event: POP_STATE}
                    if isinstance(node_test, DocumentNodeTest):
                        self._state_table[START_STATE][start_event] = newest_state
                    else:
                        for state in self._state_table:
                            self._state_table[state][start_event] = newest_state
                    self._hook_states[newest_state] = start_event
                    self._push_states.append(newest_state)
                    #if attrib_test:
                        #Because we want to treat @X as */@X
                    #    newest_state += 1
                    #    start_event = (1, ATTRIB, None)
                    #    end_event = (0, ATTRIB, None)
                    #    self._state_table[newest_state] = {end_event: newest_state - 1}
                    #    self._state_table[newest_state -1][start_event] = newest_state
                else:
                    if attrib_test:
                        attribute_test_state = newest_state
                    self._state_table[newest_state] = {end_event: parent_start_element_state}
                    self._state_table[parent_start_element_state][start_event] = newest_state
                for state in self._hook_states:
                    self._state_table[newest_state][self._hook_states[state]] = state
                
                start_element_state = newest_state
                #complete_state = top_state #The state representing completion of an XPattern
                if step_count and not ancestor and not isinstance(node_test, PrincipalTypeTest):
                    #Insert a state, which handles any child element
                    #Not explicitly matching some other state (so that
                    #/a/b/c is not a mistaken match for XPattern /a/c)
                    start_event = (1, ANY, IMPLICIT)
                    end_event = (0, ANY, IMPLICIT)
                    newest_state += 1
                    self._state_table[newest_state] = {}
                    self._state_table[parent_start_element_state][start_event] = newest_state
                    self._state_table[newest_state][end_event] = parent_start_element_state
                    self._ignored_subtree_states.append(newest_state)
                    #self._hook_states[newest_state] = start_event
                    for state in self._hook_states:
                        self._state_table[newest_state][self._hook_states[state]] = state
                parent_start_element_state = start_element_state
            self._live_states[start_element_state] = repr_xp
            if attrib_test:
                self._attrib_tests[attribute_test_state] = Compile('@'+repr(node_test))
        self._state = START_STATE
        self.entering_xpatterns = []
        self.leaving_xpatterns = []
        self.current_xpatterns = []
        self.tree_depth = 0
        self.depth_marks = []
        self.state_stack = []
        #print self._state_table; print self._live_states; print self._push_states; print self._attrib_tests
        return

    def event(self, is_start, ns, local):
        """
        Register an event and effect any state transitions
        found in the state table
        """
        #We only have a chunk ready for the handler in
        #the explicit case below
        self.entering_xpatterns = []
        self.leaving_xpatterns = []
        self.attribute_test = None
        self.tree_depth += is_start and 1 or -1
        #print "event", (is_start, ns, local), self._state, self.state_stack, self.tree_depth, self.depth_marks
        #An end event is never significant unless we know we're expecting it
        if not is_start and self.depth_marks and self.tree_depth != self.depth_marks[-1]:
            return self._state
        lookup_from = self._state_table[self._state]
        #FIXME: second part should be an element node test "*", should not match, say, start document
        if not lookup_from.has_key((is_start, ns, local)) and (ns, local) == (None, None):
            return self._state

        def process_transition(new_state):
            if new_state != self._state:
                if is_start and new_state in self._push_states:
                    self.state_stack.append(self._state)
                elif new_state == POP_STATE:
                    new_state = self.state_stack.pop()
                self._state = new_state
                if is_start:
                    self.depth_marks.append(self.tree_depth - 1)
                elif self.depth_marks:
                    self.depth_marks.pop()
            if is_start:
                attrib_state = self._state_table[new_state].get((1, ATTRIB, None))
                if attrib_state is not None:
                    self.attribute_test = self._attrib_tests[attrib_state]

        if ((is_start, ns, local) in lookup_from) or ((is_start, ns, ANY) in lookup_from):
            try:
                new_state = lookup_from[(is_start, ns, local)]
            except KeyError:
                new_state = lookup_from[(is_start, ns, ANY)]
            if (new_state in self._live_states):
                #Entering a defined XPattern chunk
                self.entering_xpatterns.append(self._live_states[new_state])
                self.current_xpatterns.append(self._live_states[new_state])
            elif (self._state in self._live_states):
                #Leaving a defined XPattern chunk
                self.leaving_xpatterns.append(self.current_xpatterns.pop())
            process_transition(new_state)
        elif (is_start, ANY, EXPLICIT) in lookup_from:
            new_state = lookup_from[(is_start, ANY, EXPLICIT)]
            if (new_state in self._live_states):
                #Entering a defined XPattern chunk
                self.entering_xpatterns.append(self._live_states[new_state])
                self.current_xpatterns.append(self._live_states[new_state])
            elif (self._state in self._live_states):
                #Leaving a defined XPattern chunk
                self.leaving_xpatterns.append(self.current_xpatterns.pop())
            process_transition(new_state)
        elif (is_start, ANY, IMPLICIT) in lookup_from:
            new_state = lookup_from[(is_start, ANY, IMPLICIT)]
            process_transition(new_state)
        else:
            #Identity transition: from a state back to itself
            process_transition(self._state)
        #print self.entering_xpatterns,self.leaving_xpatterns,self.current_xpatterns
        return self._state

    def status(self):
        """
        1 if currently within an XPattern, 0 if not
        Calling code might also want to just check
        self.current_xpatterns directly
        """
        return not not self.current_xpatterns


class xpattern_state_manager:
    """
    And aggregation of multiple state machines, one for each registered pattern
    """
    PARSER = XPatternParser.new()
    
    def __init__(self, xpatterns, nss):
        if not hasattr(xpatterns[0], "match"):
            self._xpatterns = [ (p, self.PARSER.parse(p)) for p in xpatterns ]
        else:
            self._xpatterns = [ (repr(xp), self.PARSER.parse(p)) for p in xpatterns ]
        self._machines = [ xpattern_state_machine(repr_xp, xp, nss) for repr_xp, xp in self._xpatterns ]
        return

    def event(self, is_start, ns, local):
        for machine in self._machines:
            machine.event(is_start, ns, local)
        #FIXME: Slow and clumsy
        self.entering_xpatterns = list(itertools.chain(*[m.entering_xpatterns for m in self._machines]))
        self.leaving_xpatterns = list(itertools.chain(*[m.leaving_xpatterns for m in self._machines]))
        self.current_xpatterns = list(itertools.chain(*[m.current_xpatterns for m in self._machines]))
        self.attribute_tests = [m.attribute_test for m in self._machines if m.attribute_test]
        #print "manager event", (self.entering_xpatterns, self.leaving_xpatterns, self.current_xpatterns)
        return

    def status(self):
        """
        1 if currently within an XPattern, 0 if not
        Calling code might also want to just check
        self.current_xpatterns directly
        """
        return not not self.current_xpatterns


class sax2dom_chunker(sax.ContentHandler):
    """
    Note: Ignores nodes prior to the document element, such as PIs and
    text nodes.  Collapses CDATA sections into plain text
    Only designed to work if you set the feature
      sax.handler.feature_namespaces
    to 1 on the parser you use.

    xpatterns - list of XPatterns.  Only portions of the
        tree within these patterns will be instantiated as DOM (as
        chunks fed to chunk_consumer in sequence)
        If None (the default, a DOM node will be created representing
        the entire tree.

    nss - a dictionary of prefix -> namespace name mappings used to
        interpret XPatterns
    
    chunk_consumer - a callable object taking a DOM node.  It will be
        invoked as each DOM chunk is prepared.
    
    domimpl - DOM implemention to build, e.g. mindom (the default)
        or cDomlette or pxdom (if you have the right third-party
        packages installed).
    
    owner_doc - for advanced uses, if you want to use an existing
        DOM document object as the owner of all created nodes.
        
    """
    def __init__(self,
                 xpatterns=None,
                 nss=None,
                 chunk_consumer=None,
                 domimpl=Domlette.implementation,
                 owner_doc=None,
                 ):
        nss = nss or {}
        #HINT: To use minidom
        #domimpl = xml.dom.minidom.getDOMImplementation()
        self._impl = domimpl
        if isinstance(xpatterns, str) or isinstance(xpatterns, unicode) :
            xpatterns = [xpatterns]
        #print xpatterns
        if owner_doc:
            self._owner_doc = owner_doc
        else:
            try:
                dt = self._impl.createDocumentType(DUMMY_DOCELEM, None, u'')
            except AttributeError:
                #Domlette doesn't need createDocumentType
                dt = None
            self._owner_doc = self._impl.createDocument(
                DUMMY_DOCELEM, DUMMY_DOCELEM, dt)
        #Create a docfrag to hold all the generated nodes.
        root_node = self._owner_doc.createDocumentFragment()
        self._nodeStack = [ root_node ]
        self.state_machine = xpattern_state_manager(xpatterns, nss)
        self._chunk_consumer = chunk_consumer
        return

    def get_root_node(self):
        """
        Only useful if the user does not register trim paths
        If so, then after SAX processing the user can call this
        method to retrieve resulting DOM representing the entire
        document
        """
        return self._nodeStack[0]

    #Overridden ContentHandler methods
    def startDocument(self):
        self.state_machine.event(1, None, None)
        return

    def endDocument(self):
        self.state_machine.event(0, None, None)
        return

    def startElementNS(self, (ns, local), qname, attribs):
        self.state_machine.event(1, ns, local)
        if not self.state_machine.status():
            for attrtest in self.state_machine.attribute_tests:
                e = self._owner_doc.createElementNS(ns, qname or local)
                for (ns, local), value in attribs.items():
                    e.setAttributeNS(ns, attribs.getQNameByName((ns, local)), value)
                #e = dummy_element(qname, attribs, self._nodeStack[0])
                matched = attrtest.evaluate(Context.Context(e))
                if matched:
                    #Feed the consumer
                    #self._nodeStack[TOP].appendChild(matched[0]) #"xml.dom.HierarchyRequestErr: Ft.Xml.cDomlette.Attr nodes cannot be a child of Ft.Xml.cDomlette.DocumentFragment nodes"
                    self._chunk_consumer(matched[0])
                    #Start over with new doc frag so old memory can be reclaimed
                    root_node = self._owner_doc.createDocumentFragment()
                    self._nodeStack = [ root_node ]
            return
            
        new_element = self._owner_doc.createElementNS(ns, qname or local)

        for (attr_ns, lname) in attribs:
            value = attribs[(attr_ns, lname)]
            if attr_ns is not None:
                attr_qname = attribs.getQNameByName((attr_ns, lname))
            else:
                attr_qname = lname
            attr = self._owner_doc.createAttributeNS(
                attr_ns, attr_qname)
            attr_qname = attribs.getQNameByName((attr_ns, lname))
            attr.value = value
            new_element.setAttributeNodeNS(attr)

        self._nodeStack.append(new_element)
        return

    def endElementNS(self, (ns, local), qname):
        self.state_machine.event(0, ns, local)
        if not self.state_machine.status():
            if (self._chunk_consumer and
                self.state_machine.leaving_xpatterns):
                #Complete the element being closed because it
                #Is the last bit of a DOM to be fed to the consumer
                new_element = self._nodeStack[TOP]
                del self._nodeStack[TOP]
                self._nodeStack[TOP].appendChild(new_element)
                #Feed the consumer
                self._chunk_consumer(self._nodeStack[0])
                #Start all over with a new doc frag so the old
                #One's memory can be reclaimed
                root_node = self._owner_doc.createDocumentFragment()
                self._nodeStack = [ root_node ]
            return
        new_element = self._nodeStack[TOP]
        del self._nodeStack[TOP]
        self._nodeStack[TOP].appendChild(new_element)
        return

    def processingInstruction(self, target, data):
        if self.state_machine.status():
            pi = self._owner_doc.createProcessingInstruction(
                target, data)
            self._nodeStack[TOP].appendChild(pi)
        return

    def characters(self, chars):
        if self.state_machine.status():
            new_text = self._owner_doc.createTextNode(chars)
            self._nodeStack[TOP].appendChild(new_text)
        return

    #Overridden LexicalHandler methods
    def comment(self, text):
        if self.state_machine.status():
            new_comment = self._owner_doc.createComment(text)
            self._nodeStack[TOP].appendChild(new_comment)
        return


from xml.sax.saxutils import XMLFilterBase
#FIXME: Set up to use actual PyXML if available
from amara.pyxml_standins import *

class normalize_text_filter(XMLFilterBase, LexicalHandler):
    """
    SAX filter to ensure that contiguous white space nodes are
    delivered merged into a single node
    """
    def __init__(self, *args):
        XMLFilterBase.__init__(self, *args)
        self._accumulator = []
        return

    def _complete_text_node(self):
        if self._accumulator:
            XMLFilterBase.characters(self, ''.join(self._accumulator))
            self._accumulator = []
        return

    def startDocument(self):
        XMLFilterBase.startDocument(self)
        return

    def endDocument(self):
        XMLFilterBase.endDocument(self)
        return

    def startElement(self, name, attrs):
        self._complete_text_node()
        XMLFilterBase.startElement(self, name, attrs)
        return

    def startElementNS(self, name, qname, attrs):
        self._complete_text_node()
        #A bug in Python 2.3 means that we can't just defer to parent, which is broken
        #XMLFilterBase.startElementNS(self, name, qname, attrs)
        self._cont_handler.startElementNS(name, qname, attrs)
        return

    def endElement(self, name):
        self._complete_text_node()
        XMLFilterBase.endElement(self, name)
        return

    def endElementNS(self, name, qname):
        self._complete_text_node()
        XMLFilterBase.endElementNS(self, name, qname)
        return

    def processingInstruction(self, target, body):
        self._complete_text_node()
        XMLFilterBase.processingInstruction(self, target, body)
        return

    def comment(self, body):
        self._complete_text_node()
        #No such thing as an XMLFilterBase.comment :-(
        #XMLFilterBase.comment(self, body)
        self._cont_handler.comment(body)
        return

    def characters(self, text):
        self._accumulator.append(text)
        return

    def ignorableWhitespace(self, ws):
        self._accumulator.append(text)
        return

    #Must be overridden because of a bug in Python 2.0 through 2.4
    #And even still in PyXML 0.8.4.  Missing "return"
    def resolveEntity(self, publicId, systemId):
        return self._ent_handler.resolveEntity(publicId, systemId)

    # Enhancement suggested by James Kew:
    # Override XMLFilterBase.parse to connect the LexicalHandler
    # Can only do this by setting the relevant property
    # May throw SAXNotSupportedException
    def parse(self, source):
        #import inspect; import pprint; pprint.pprint(inspect.stack())
        self._parent.setProperty(property_lexical_handler, self)
        # Delegate to XMLFilterBase for the rest
        XMLFilterBase.parse(self, source)
        return


#
# From xml.dom
#

#ELEMENT_NODE                = 1
#ATTRIBUTE_NODE              = 2
#TEXT_NODE                   = 3
#CDATA_SECTION_NODE          = 4
#ENTITY_REFERENCE_NODE       = 5
#ENTITY_NODE                 = 6
#PROCESSING_INSTRUCTION_NODE = 7
#COMMENT_NODE                = 8
#DOCUMENT_NODE               = 9
#DOCUMENT_TYPE_NODE          = 10
#DOCUMENT_FRAGMENT_NODE      = 11
#NOTATION_NODE               = 12




# -*- coding: utf-8 -*-
#Tests examples given in the manual

import unittest
import amara
from amara import binderytools
from Ft.Xml import InputSource
from Ft.Lib import Uri


class TestBase(unittest.TestCase):
    def setUp(self):
        self.isrc_factory = InputSource.DefaultFactory
        return

    def _input_source_for_file(self, fname):
        #Create a URI from a filename the right way
        file_uri = Uri.OsPathToUri(fname, attemptAbsolute=1)
        isrc = self.isrc_factory.fromUri(file_uri)
        return isrc

    def _input_source_for_string(self, text, uri="urn:bogus:dummy"):
        isrc = self.isrc_factory.fromString(text, uri)
        return isrc


class TestMJ030815(TestBase):
    XML = """\
<birds country='germany' >
    <raptor endangered='yes' wingspan='240cm'>
        <english_name>White-tailed Eagle</english_name>
        <german_name>Seeadler</german_name>
        <prey>birds</prey>
        <prey>fish</prey>
    </raptor>
    <raptor endangered='no' wingspan='80cm'>
        <english_name>Kestrel</english_name>
        <german_name>Turmfalke</german_name>
        <prey>mice</prey>
    </raptor>
</birds>
"""
    
    def testSpuriousPrints(self):
        #isrc = self._input_source_for_string(self.XML)
        #binder = anobind.binder()
        #binding = binder.read_xml(isrc)
        binding = amara.parse(self.XML)

        output = []
        for raptor in binding.birds.raptor:
            output.append(raptor.wingspan)
            output.append(unicode(raptor.german_name))

        for raptor in binding.xml_xpath(u'/birds/raptor'):
            output.append(raptor.wingspan)
            output.append(unicode(raptor.german_name))

        self.assertEqual(output, [u'240cm', u'Seeadler', u'80cm', u'Turmfalke', u'240cm', u'Seeadler', u'80cm', u'Turmfalke'])
        return


class TestMI030914(TestBase):
    #From private message, sender's name withheld
    XML = """\
<?xml version="1.0" encoding="UTF-8" ?> 
<p1:spokeBookEntry p2:type="p1:SpokeBookEntryType" nonVcf="true" isInvited="false" isSuppressed="false" href="/member/510379/spokebook/512367" inPrivacyFence="false" privacyLevel="public" xmlns:p1="http://ns.spoke.com/namespace/platform/1.0" xmlns:p2="http://www.w3.org/2001/XMLSchema-instance">
  <p1:person href="/person/512373" /> 
  <p1:sor>26.21035385131836</p1:sor> 
  <p1:sorDelta>0</p1:sorDelta>
  <p1:firstName>Max</p1:firstName> 
  <p1:lastName>Moolah</p1:lastName> 
  <p1:job p2:type="p1:Job" jobID="5866450" isPrimary="true">
    <p1:company href="/company/18711" /> 
    <p1:jobTitle>Chief Executive Officer</p1:jobTitle> 
    <p1:companyName>Juggernaut Holdings</p1:companyName> 
  </p1:job>
  <p1:job p2:type="p1:Job" jobID="421504" isPrimary="false">
    <p1:company href="/company/18711" /> 
    <p1:jobTitle /> 
    <p1:companyName>The Domi Nation, PLC</p1:companyName> 
  </p1:job>
  <p1:address p2:type="p1:StreetAddress" locationID="850151" for="home">
    <p1:state /> 
    <p1:postCode>94403</p1:postCode> 
  </p1:address>
</p1:spokeBookEntry>
"""
 
    def testSimpleLoops(self):
        prefixes = {
            'p1': 'http://ns.spoke.com/namespace/platform/1.0'
            }
        doc = amara.parse(self.XML, prefixes=prefixes)
        #isrc = self._input_source_for_string(self.XML)
        #binder = anobind.binder(namespaces=namespaces)
        #doc = binder.read_xml(isrc)

        output = []
        for r in doc.spokeBookEntry:
            output.append(unicode(r.firstName))
            output.append(unicode(r.lastName))

        for r in doc.spokeBookEntry.xml_xpath(u'p1:job'):
            output.append(unicode(r.companyName))

        self.assertEqual(output, [u'Max', u'Moolah', u'Juggernaut Holdings', u'The Domi Nation, PLC'])
        #self.assertEqual(output, [u'Mary', u'Walker', u'Satmetrix', u'CustomerCast'])
        return

class TestTL030315(TestBase):
    #From Tom Lazar
    def testPiSerialize(self):
        XML = """\
<?xml version="1.0" encoding="UTF-8"?><?xml-stylesheet type="text/css"  
href="../css/services.css"?>
<!--
<!DOCTYPE services
   PUBLIC "- -//tomster.org//DTD Services//DE" "file:../dtd/services.dtd">
   -->
<services>
  <client id="1021">
    <service billing-id="1314" location="self" performer="tomster" type="isp">
      <description>...</description>
        <date>2005-01-31</date>
        <count>3</count>
        <amount>2.00</amount>
      </service>
      [...]
  </client>
</services>
"""

        EXPECTED = '<?xml version="1.0" encoding="UTF-8"?>\n<?xml-stylesheet type="text/css"  \nhref="../css/services.css"?><!--\n<!DOCTYPE services\n   PUBLIC "- -//tomster.org//DTD Services//DE" "file:../dtd/services.dtd">\n   --><services>\n  <client id="1021">\n    <service performer="tomster" type="isp" location="self" billing-id="1314">\n      <description>...</description>\n        <date>2005-01-31</date>\n        <count>3</count>\n        <amount>2.00</amount>\n      </service>\n      [...]\n  </client>\n</services>'
        doc = amara.parse(XML)
        self.assertEqual(doc.xml(), EXPECTED)
        return


class TestTL030315(TestBase):
    #From Tom Lazar
    def testMerge(self):
        XML = """\
<?xml version="1.0" encoding="utf-8"?>
<?xml-stylesheet type="text/css" href="../css/services.css"?>
<!--
<!DOCTYPE services PUBLIC "- -//tomster.org//DTD Services//DE" "file:../dtd/services.dtd">
-->
<services>
  <client id="1023">
    <service billing_id="2" performer="tomster" type="travel" location="client">
      <description>ausserhalb Berlins</description>

      <date>2005-03-04</date>

      <count>1</count>

      <amount>36</amount>
    </service>

    <service billing_id="2" performer="tomster" type="workunit" location="client">
      <description>Umbau Serverplatte in G4 mit funktionierendem optischem
      Laufwerk, Neuvergabe Adminpasswort, Umzug der Rechenanlage in Wattstr.
      17, Aufbau und Test dort</description>

      <date>2005-03-04</date>

      <count>24</count>

      <amount>10</amount>
    </service>

    <service billing_id="2" performer="tomster" type="admin" location="client">
      <description>Telefonsupport</description>

      <date>2005-03-26</date>

      <count>1</count>

      <amount>10</amount>
    </service>
  </client>

  <client id="1035">
    <service billing_id="1" performer="tomster" type="isp" location="self">
      <description>Domainhosting für die domain <b>am-viscom.de</b> für den
      Zeitraum vom 31.9.2004 bis 31.12.2004</description>

      <date>2005-01-31</date>

      <count>3</count>

      <amount>2.00</amount>
    </service>

    <service billing_id="1" performer="tomster" type="isp" location="self">
      <description>500 Mb Speicherplatz für <b>annette</b> für den Zeitraum
      vom 31.9.2004 bis 31.12.2004</description>

      <date>2005-01-31</date>

      <count>3</count>

      <amount>5</amount>
    </service>

    <service billing_id="1" performer="tomster" type="isp" location="self">
      <description>IMAP Dienst inkl. täglichem Backup für das Konto
      <b>annette</b> für den Zeitraum vom 31.9.2004 bis
      31.12.2004</description>

      <date>2005-01-31</date>

      <count>3</count>

      <amount>5.00</amount>
    </service>

    <service billing_id="1" performer="tomster" type="isp" location="self">
      <description>100 Mb Speicherplatz für <b>loic</b> für den Zeitraum vom
      31.9.2004 bis 31.12.2004</description>

      <date>2005-01-31</date>

      <count>3</count>

      <amount>1</amount>
    </service>

    <service billing_id="0" performer="tomster" type="webdev" location="self">
      <description>IMAP Dienst inkl. täglichem Backup für das Konto
      <b>loic</b> für den Zeitraum vom 31.9.2004 bis 31.12.2004</description>

      <date>2005-01-31</date>

      <count>3</count>

      <amount>5.00</amount>
    </service>
  </client>

<client id="1063">
    <service billing_id="0" performer="ekke" type="admin" location="client">
      <description>Router zurückgesetzt, RAM-Bestellung abgewickelt, backups
      überprüft</description>
      <date>2005-02-02</date>
      <count>0,5</count>
      <amount>50</amount>
    </service>
    <service billing_id="0" performer="ekke" type="admin" location="client">
      <description>G5 iMac komplett neu eingerichtet; Benutzer übertragen,
      diverse Programme installiert, Schriften, Drucker; Druckerproblem
      Xerox/Archifile A3 behoben; Farbkopierer Varianten kurz durchgesprochen,
      Mailimport begonnen; Powerbook G4 komplett neuinstallliert wegen
      DongleProblem LazyJack; funktioniert nun; Benutzer übertragen, Drucker
      und Programme nachinstalliert</description>
      <date>2005-02-09</date>
      <count>6</count>
      <amount>50</amount>
    </service>
    <service billing_id="0" performer="ekke" type="admin" location="client">
      <description>ap 22 Adobe Suite installiert, diverse Programme, Mail
      importiert, Adressen importiert</description>
      <date>2005-02-10</date>
      <count>1,5</count>
      <amount>50</amount>
    </service>
    <service billing_id="0" performer="ekke" type="admin" location="client">
      <description>ag Mail repariert, ap22 Plotter nachinstalliert, Programme
      installiert, 2 iMacs Festplatten ausgetauscht, 3 Systemupdates
      (ap20-22), Druckproblem archifile behoben (hpLJ Buero), 2 x RAM
      installiert</description>
      <date>2005-02-16</date>
      <count>4,2</count>
      <amount>50</amount>
    </service>
    <service billing_id="0" performer="ekke" type="admin" location="client">
      <description>Systemupdates 10.3.8 eingespielt 3 Maschinen,
      Securityupdate 2005-002 eingespielt auf 7 Maschinen, FontShop wegen
      Schriftenproblem kontaktiert, ap21 Rechteproblem behoben, Renderingfrage
      geklärt, ap20 Keynote, Entsorgung der Monitore besprochen, backups
      überprüft, ap9, ap20 Software nachinstalliert (Office)</description>
      <date>2005-02-23</date>
      <count>2,75</count>
      <amount>50</amount>
    </service>
    <service billing_id="0" performer="ekke" type="admin" location="client">
      <description>3 Rechner rechte repariert/Acrobat-Problem behoben,
      Zugriffsrechte für "Projekte ausgelagert" auf dem server geändert,
      Backupset 20GB-Spiegel wegen Platzproblemen angepaßt, Test-Rückspielung
      eine Files aus backupset zur Kontrolle mit Überprüfung (Start der
      LazyJack-Datenbank von der wiederhergestellten Datei)</description>
      <date>2005-02-34</date>
      <count>2,5</count>
      <amount>50</amount>
    </service>
  </client>
  <client id="2055">
    <service billing_id="0" performer="ekke" type="admin" location="client">
      <description>111111</description>
      <date>2005-02-66</date>
      <count>6</count>
      <amount>50</amount>
    </service>
    <service billing_id="0" performer="ekke" type="admin" location="client">
      <description>2222222</description>
      <date>2005-02-66</date>
      <count>6</count>
      <amount>50</amount>
    </service>
    <service billing_id="0" performer="ekke" type="admin" location="client">
      <description>3333</description>
      <date>2005-02-66</date>
      <count>6</count>
      <amount>50</amount>
    </service>
    <service billing_id="0" performer="ekke" type="admin" location="client">
      <description>44444</description>
      <date>2005-02-66</date>
      <count>6</count>
      <amount>50</amount>
    </service>
    <service billing_id="0" performer="ekke" type="admin" location="client">
      <description>55555</description>
      <date>2005-02-66</date>
      <count>6</count>
      <amount>50</amount>
    </service>
    <service billing_id="0" performer="ekke" type="admin" location="client">
      <description>666666</description>
      <date>2005-02-66</date>
      <count>6</count>
      <amount>50</amount>
    </service>
    <service billing_id="0" performer="ekke" type="admin" location="client">
      <description>777777</description>
      <date>2005-02-66</date>
      <count>6</count>
      <amount>50</amount>
    </service>
  </client>
</services>"""

        EXPECTED = ''
        doc1 = amara.parse(XML)
        doc2 = amara.parse(XML)
        #self.assertEqual(doc.xml(), EXPECTED)
        for client in doc1.services.client:
            for service_node in client.service:
                doc2.services.client.xml_append(service_node)
        return

XMLDECL = '<?xml version="1.0" encoding="UTF-8"?>\n'

class TestJN050418(TestBase):
    #From Jamie Norrish
    def getEntryList(self, doc):
        entryList = []
        for entry in doc.entries.entry:
            entryList.append(unicode(entry))
        return entryList

    def removeEntryByID(self, doc, entryID):
        entry = doc.entries.xml_xpath('entry[@id="%s"]' % unicode(entryID))[0]
        #index = entry.xml_index_on_parent()
        doc.entries.xml_remove_child(entry)

    def makeEntry(self, param):
        doc = amara.create_document()
        doc.xml_append(doc.xml_create_element(u'entry',
                                       attributes={'id': unicode(param)}))
        doc.entry.xml_children.append(unicode(param))
        return doc.entry

    def testRepeatedReplace(self):
        EXPECTED = '<entries><entry id="entry1">entry1</entry><entry id="entry2">entry2</entry></entries>'
        doc = amara.parse('<entries></entries>')
        entry1 = self.makeEntry(u'entry1')
        entry2 = self.makeEntry(u'entry2')
        doc.entries.xml_append(entry1)
        doc.entries.xml_append(entry2)
        self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
        self.assertEqual(self.getEntryList(doc), [u'entry1', u'entry2'])
        for i in range(3):
            entry = self.makeEntry(u'entry2')
            self.removeEntryByID(doc, u'entry2')
            doc.entries.xml_append(entry)
            #self.assertEqual(doc.)
            #self.assertEqual(doc.)
            self.assertEqual(doc.xml(), XMLDECL+EXPECTED)
            self.assertEqual(self.getEntryList(doc), [u'entry1', u'entry2'])

class TestJN050418_1(TestJN050418):
    #First simplification of makeEntry
    def makeEntry(self, param):
        doc = amara.create_document()
        doc.xml_append(doc.xml_create_element(u'entry',
                                       attributes={'id': unicode(param)}))
        doc.entry.xml_append(unicode(param))
        return doc.entry

class TestJN050418_2(TestJN050418):
    #Second simplification of makeEntry
    def makeEntry(self, param):
        doc = amara.create_document()
        doc.xml_append(doc.xml_create_element(u'entry',
                                       attributes={'id': unicode(param)},
                                       content=unicode(param)))
        return doc.entry

class TestJN050418_3(TestJN050418):
    #Third simplification of makeEntry
    def makeEntry(self, param):
        doc = amara.create_document(u'entry',
                                           attributes={'id': unicode(param)},
                                           content=unicode(param))
        return doc.entry

class TestAlain050601:
    XML = """\
<html><head>
<title>Document</title>
</head>
<body>
<script type="text/javascript">
//<![CDATA[
function matchwo(a,b)
{
if (a < b && a > 0) then
   {
   return 1
   }
}
//]]>
</script>
</body>
</html>
"""
    
    def testCdataOutput1(self):
        doc = amara.parse(self.XML)
        import sys
        from Ft.Xml.Xslt.XmlWriter import CdataSectionXmlWriter
        from Ft.Xml.Xslt.OutputParameters import OutputParameters
        doc = parse(text)
        op = OutputParameters()
        op.cdataSectionElements=[(None, u'script')]
        w = CdataSectionXmlWriter(op, sys.stdout)
        out = doc.xml(writer=w)
        #Way too simple.  Fix this test
        self.assert_(out.find('CDATA'))
        return

    def testCdataOutput2(self):
        doc = amara.parse(self.XML)
        out = doc.xml(cdataSectionElements=[u'script']) 
        #Way too simple.  Fix this test
        self.assert_(out.find('CDATA'))
        return

class TestLM050928(TestBase):
    #Luis Morillas reported make_element bug
    def testCreateElement17(self):
        import amara
        import string
        doc = amara.create_document(u'abc')
        for n, l in enumerate(string.letters):
            newLetter = doc.xml_create_element(
                u'letter',
                attributes={'id': unicode(n+1)},
                content= unicode(l)
                )
            doc.abc.xml_append(newLetter)
            #print 'adding', l , '...'
            #if n == 17: break # with 18 elements works fine :-/
        self.assertEqual(len(doc.abc.letter), len(string.letters))
        return


class TestA051123(TestBase):
    #"ancaster" reported empty document create bug
    def testCreateEmptyDoc(self):
        import amara
        doc = amara.create_document()
        doc.xml_append(doc.xml_create_element(u"foo"))
        self.assertEqual(len(doc.xml_children), 1)
        self.assertEqual(len(doc.xml_xpath(u'/')), 1)
        return


class TestIP060124(unittest.TestCase):
    #Ian Phillips reported mangling with rules
    XML = """\
<?xml version="1.0" encoding="UTF-8"?>
<repository xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
            xmlns="http://www.tibco.com/xmlns/repo/types/2002">
\t<globalVariables>
\t\t<globalVariable>
\t\t\t<name>ADBScriptFileDir</name>
\t\t\t<value>C:\\tibco\\adapter\\adadb\\5.1\\sql</value>
\t\t\t<deploymentSettable>true</deploymentSettable>
\t\t\t<serviceSettable>false</serviceSettable>
\t\t\t<type>String</type>
\t\t\t<modTime>1131636444541</modTime>
\t\t</globalVariable>
\t\t<globalVariable>
\t\t\t<name>Billing</name>
\t\t\t<value>Geneva</value>
\t\t\t<deploymentSettable>false</deploymentSettable>
\t\t\t<serviceSettable>false</serviceSettable>
\t\t\t<type>String</type>
\t\t\t<modTime>1131636444541</modTime>
\t\t</globalVariable>
\t</globalVariables>
</repository>
    """
    MYPREFIXES = { 'gv' : 'http://www.tibco.com/xmlns/repo/types/2002' }
    def testRuleset1(self):
        my_rules = [
            binderytools.ws_strip_element_rule(),
            #binderytools.omit_element_rule( u'gv:deplymentSettable' ),
            #binderytools.omit_element_rule( u'gv:serviceSettable' ),
            #binderytools.omit_element_rule( u'gv:modTime' ),
            #binderytools.simple_string_element_rule( u'gv:name' ),
            #binderytools.simple_string_element_rule( u'gv:value' ),
            #binderytools.simple_string_element_rule( u'gv:type' )
            ]

        doc = amara.parse(self.XML, rules=my_rules, prefixes=self.MYPREFIXES)
        EXPECTED = '<globalVariable xmlns="http://www.tibco.com/xmlns/repo/types/2002"><name>ADBScriptFileDir</name><value>C:\\tibco\\adapter\\adadb\\5.1\sql</value><deploymentSettable>true</deploymentSettable><serviceSettable>false</serviceSettable><type>String</type><modTime>1131636444541</modTime></globalVariable>'
        e = doc.repository.globalVariables.globalVariable[0]
        actual = e.xml()
        self.assertEqual(EXPECTED, actual)
        self.assertEqual(len(e.xml_children), 6)
        self.assertEqual(len(e.xml_xpath(u'.//gv:value')), 1)
        return

    def testRuleset2(self):
        my_rules = [
            binderytools.ws_strip_element_rule(),
            binderytools.omit_element_rule( u'gv:deploymentSettable' ),
            ]

        doc = amara.parse(self.XML, rules=my_rules, prefixes=self.MYPREFIXES)
        EXPECTED = '<globalVariable xmlns="http://www.tibco.com/xmlns/repo/types/2002"><name>ADBScriptFileDir</name><value>C:\\tibco\\adapter\\adadb\\5.1\sql</value><serviceSettable>false</serviceSettable><type>String</type><modTime>1131636444541</modTime></globalVariable>'
        e = doc.repository.globalVariables.globalVariable[0]
        actual = e.xml()
        self.assertEqual(EXPECTED, actual)
        self.assertEqual(len(e.xml_children), 5)
        self.assertEqual(len(e.xml_xpath(u'.//gv:name')), 1)
        self.assertEqual(len(e.xml_xpath(u'.//gv:value')), 1)
        return

    def testRuleset3(self):
        my_rules = [
            binderytools.ws_strip_element_rule(),
            binderytools.omit_element_rule( u'gv:deploymentSettable' ),
            binderytools.simple_string_element_rule( u'gv:name' ),
            ]

        doc = amara.parse(self.XML, rules=my_rules, prefixes=self.MYPREFIXES)
        EXPECTED = '<globalVariable xmlns="http://www.tibco.com/xmlns/repo/types/2002" name="ADBScriptFileDir"><value>C:\\tibco\\adapter\\adadb\\5.1\sql</value><serviceSettable>false</serviceSettable><type>String</type><modTime>1131636444541</modTime></globalVariable>'
        e = doc.repository.globalVariables.globalVariable[0]
        actual = e.xml()
        self.assertEqual(EXPECTED, actual)
        self.assertEqual(len(e.xml_children), 4)
        self.assertEqual(len(e.xml_xpath(u'.//gv:name')), 0)
        self.assertEqual(len(e.xml_xpath(u'.//gv:value')), 1)
        return

    def testRuleset4(self):
        my_rules = [
            binderytools.ws_strip_element_rule(),
            binderytools.omit_element_rule( u'gv:deploymentSettable' ),
            binderytools.omit_element_rule( u'gv:serviceSettable' ),
            binderytools.omit_element_rule( u'gv:modTime' ),
            binderytools.simple_string_element_rule( u'gv:name' ),
            binderytools.simple_string_element_rule( u'gv:value' ),
            binderytools.simple_string_element_rule( u'gv:type' )
            ]

        doc = amara.parse(self.XML, rules=my_rules, prefixes=self.MYPREFIXES)
        EXPECTED = '<globalVariable xmlns="http://www.tibco.com/xmlns/repo/types/2002" type="String" name="ADBScriptFileDir" value="C:\\tibco\\adapter\\adadb\\5.1\\sql"/>'
        e = doc.repository.globalVariables.globalVariable[0]
        actual = e.xml()
        self.assertEqual(EXPECTED, actual)
        self.assertEqual(len(e.xml_children), 0)
        self.assertEqual(len(e.xml_xpath(u'.//gv:name')), 0)
        self.assertEqual(len(e.xml_xpath(u'.//gv:value')), 0)
        return


class TestKS060616(TestBase):
    #Kai Schmitte reported xml_clear() bug
    def testCreateEmptyDoc(self):
        import amara
        xml = '<root><Child1><child2><entry user = "everyone"/><entry user = "site1"/></child2></Child1></root>'
        doc = amara.parse(xml)
        doc.root.Child1.child2.xml_clear()
        EXPECTED = '<?xml version="1.0" encoding="UTF-8"?>\n<root><Child1><child2/></Child1></root>'
        self.assertEqual(len(doc.root.Child1.child2.xml_children), 0)
        self.assertEqual(doc.xml(), EXPECTED)
        return


ANS = u'http://www.w3.org/2005/Atom'
XNS = u'http://www.w3.org/1999/xhtml'

class TestElarson061209(TestBase):
    #Eric Larson was having trouble creating Atom entries with XHTML
    id = u'urn:x-bogus:1'
    updated = u'2006-12-10T02:40:45.343000Z'
    title = u'XSL is Fun'
    expected = """<?xml version="1.0" encoding="UTF-8"?>
<entry xmlns="http://www.w3.org/2005/Atom"><id>urn:x-bogus:1</id><title>XSL is Fun</title><updated>2006-12-10T02:40:45.343000Z</updated><content type="xhtml"><div xmlns="http://www.w3.org/1999/xhtml">
      <div class="document" id="xsl-is-fun">
        <p>The past couple months have been a lot of fun. I have been working with my blog, which has become focused around XML.</p></div>
    </div></content></entry>"""
    
    def testCreateAtom1(self):
        content = '\n      <div class="document" id="xsl-is-fun">\n        <p>The past couple months have been a lot of fun. I have been working with my blog, which has become focused around XML.</p></div>\n    '
        e = amara.create_document()
        e.xml_append(e.xml_create_element(u'entry', ANS))
        e.entry.xml_append(e.xml_create_element(u'id', ANS, content=self.id))
        e.entry.xml_append(e.xml_create_element(u'title', ANS, content=self.title))
        e.entry.xml_append(e.xml_create_element(u'updated', ANS, content=self.updated))
        e.entry.xml_append(e.xml_create_element(u'content', ANS, attributes={u'type' : u'xhtml'}))
        e.entry.content.xml_append(e.xml_create_element(u'div', XNS))
        e.entry.content.div.xml_append_fragment(content)
        self.assertEqual(e.xml(), self.expected)
        
        #e.entry.content.div.div.xmlnsUri = XNS
        return


if __name__ == '__main__':
    unittest.main()




#Test garbage collection
# Resources:
#   http://www.skymind.com/~ocrow/python_string/
#   http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/286222

import unittest
import os
import gc
from cStringIO import StringIO
import commands, os
import amara
from Ft.Xml import InputSource
from Ft.Lib import Uri

SCALE = 1000
PID = os.getpid()
PS_COMMAND = 'ps up '


class TestPushbindLoopPatterns(unittest.TestCase):
    def setUp(self):
        self.bigdoc1 = ["<A>"]
        self.bigdoc1.extend(["<B/>"]*(SCALE*100))
        self.bigdoc1.extend(["</A>"])
        self.bigdoc1 = ''.join(self.bigdoc1)
        #len(self.bigdoc1) is 400007 for SCALE = 1000
        gc.enable()
        return

    def testPushbindGC(self):
        #Use a simplistic trend watcher to make sure that memory
        #Usage is not growing unreasonably
        #Cum is a measure of the cumulative growth trend in GC profile
        cum = 0.0
        elems = amara.pushbind(self.bigdoc1, u"B")
        init_gc = gc.collect()
        #print init_gc
        e = elems.next()
        iter1_gc = gc.collect()
        #print iter1_gc
        for i in xrange(100):
            e = elems.next()
            iter_gc = gc.collect()
            #print iter_gc
            try:
                increase = iter_gc - old_iter_gc
                #Uncomment the following lines for more info
                #if increase > 0:
                #    print "Increase ", increase, "(cumulative)", cum
                cum += increase
            except NameError:
                pass
            old_iter_gc = iter_gc
            self.assert_(cum/SCALE < 0.01)
        return

    def testPushbindSystemMem(self):
        try:
            int(commands.getoutput(PS_COMMAND + `PID`).split()[15])
        except:
            #Test won't work on this platform
            #FIXME: improve/broaden the test
            pass
        prev_size = start_size = ps_stat()
        #elems = amara.pushbind(self.bigdoc1, u"B")
        runs = 100
        for i in xrange(runs):
            value = unicode(amara.pushbind('<title>title</title>', 'title').next())
            #Parse('<title>title</title>')
            gc.collect()
            size = ps_stat()
            growth = (size - prev_size)*100.0/size
            self.assert_(growth < 2.5, 'Greater than 2.5%% virtual memory size growth detected after one run (%i%%)'%growth)
            prev_size = size
        growth = (size - start_size)*100.0/size
        self.assert_(growth < 10, 'Greater than 10%% virtual memory size growth detected over %i runs (%i%%)'%(runs, growth))
        return


    def testParseSystemMem(self):
        try:
            int(commands.getoutput(PS_COMMAND + `PID`).split()[15])
        except:
            #Test won't work on this platform
            #FIXME: improve/broaden the test
            pass
        prev_size = start_size = ps_stat()
        #elems = amara.pushbind(self.bigdoc1, u"B")
        runs = 100
        for i in xrange(runs):
            amara.parse('<title>title</title>')
            gc.collect()
            size = ps_stat()
            growth = (size - prev_size)*100.0/size
            self.assert_(growth < 2.5, 'Greater than 2.5%% virtual memory size growth detected after one run (%i%%)'%growth)
            prev_size = size
        growth = (size - start_size)*100.0/size
        self.assert_(growth < 10, 'Greater than 10%% virtual memory size growth detected over %i runs (%i%%)'%(runs, growth))
        return


def ps_stat():
    ps = commands.getoutput(PS_COMMAND + `PID`)
    process_size = int(ps.split()[15])
    #process_size = ps.split()[15]
    return process_size


if __name__ == '__main__':
    unittest.main()




#Test examples in http://www.xml.com/pub/a/2003/11/12/schematron.html
import sys
import unittest
from util import run_stron


ENCODING = 'utf-8'


#Note: fixed a typo in the article: missing parenthesis after 'Male' in assert test
STRON1 = '''\
<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://purl.oclc.org/dsdl/schematron">
    <pattern name="Check structure">
        <rule context="Person">
            <assert test="@Title">The element Person must have a Title attribute.</assert>
            <assert test="count(*) = 2 and count(Name) = 1 and count(Gender) = 1">The element Person should have the child elements Name and Gender.</assert>
            <assert test="*[1] = Name">The element Name must appear before element Age.</assert>
            <assert test="(@Title = 'Mr' and Gender = 'Male') or @Title != 'Mr'">If the Title is "Mr" then the gender of the person must be "Male".</assert>
        </rule>
    </pattern>
</schema>
'''

STRON2 = '''\
<?xml version="1.0" encoding="UTF-8"?>
<sch:schema xmlns:sch="http://purl.oclc.org/dsdl/schematron">
    <sch:ns uri="http://www.topologi.com/example" prefix="ex"/>
    <sch:pattern name="Check structure">
        <sch:rule context="ex:Person">
            <sch:assert test="@Title">The element Person must have a Title attribute</sch:assert>
            <sch:assert test="count(ex:*) = 2 and count(ex:Name) = 1 and count(ex:Gender) = 1">The element Person should have the child elements Name and Gender.</sch:assert>
            <sch:assert test="ex:*[1] = ex:Name">The element Name must appear before element Gender.</sch:assert>
        </sch:rule>
    </sch:pattern>
    <sch:pattern name="Check co-occurrence constraints">
        <sch:rule context="ex:Person">
            <sch:assert test="(@Title = 'Mr' and ex:Gender = 'Male') or @Title != 'Mr'">If the Title is "Mr" then the gender of the person must be "Male".</sch:assert>
        </sch:rule>
    </sch:pattern>
</sch:schema>
'''

STRON3 = '''\
<?xml version="1.0" encoding="UTF-8"?>
<sch:schema xmlns:sch="http://purl.oclc.org/dsdl/schematron">
  <sch:pattern id="Time" abstract="true">
    <sch:rule context="$time">
        <sch:assert test="$hour>=0 and $hour&lt;=23">The hour must be a value between 0 and 23.</sch:assert>
        <sch:assert test="$minute>=0 and $minute&lt;=59">The minutes must be a value between 0 and 59.</sch:assert>
        <sch:assert test="$second>=0 and $second&lt;=59">The seconds must be a value between 0 and 23.</sch:assert>
    </sch:rule>
  </sch:pattern>

  <sch:pattern name="SingleLineTime" is-a="Time">
    <sch:param formal="time" actual="time"/>
    <sch:param formal="hour" actual="number(substring(.,1,2))"/>
    <sch:param formal="minute" actual="number(substring(.,4,2))"/>
    <sch:param formal="second" actual="number(substring(.,7,2))"/>
  </sch:pattern>
</sch:schema>
'''

STRON4 = '''\
<?xml version="1.0" encoding="UTF-8"?>
<sch:schema xmlns:sch="http://purl.oclc.org/dsdl/schematron">
  <sch:pattern id="Time" abstract="true">
    <sch:rule context="$time">
        <sch:assert test="$hour>=0 and $hour&lt;=23">The hour must be a value between 0 and 23.</sch:assert>
        <sch:assert test="$minute>=0 and $minute&lt;=59">The minutes must be a value between 0 and 59.</sch:assert>
        <sch:assert test="$second>=0 and $second&lt;=59">The seconds must be a value between 0 and 23.</sch:assert>
    </sch:rule>
  </sch:pattern>

  <sch:pattern name="MultiLineTime" is-a="Time">
    <sch:param formal="time" actual="time"/>
    <sch:param formal="hour" actual="hour"/>
    <sch:param formal="minute" actual="minute"/>
    <sch:param formal="second" actual="second"/>
  </sch:pattern>
</sch:schema>
'''

STRON5 = '''\
<?xml version="1.0" encoding="UTF-8"?>
<sch:schema xmlns:sch="http://purl.oclc.org/dsdl/schematron">
  <sch:pattern id="Time" abstract="true">
    <sch:rule context="$time">
        <sch:assert test="$hour>=0 and $hour&lt;=23">The hour must be a value between 0 and 23.</sch:assert>
        <sch:assert test="$minute>=0 and $minute&lt;=59">The minutes must be a value between 0 and 59.</sch:assert>
        <sch:assert test="$second>=0 and $second&lt;=59">The seconds must be a value between 0 and 23.</sch:assert>
    </sch:rule>
  </sch:pattern>

  <sch:pattern name="Dummy pattern">
    <sch:rule context="dummy">
      <sch:assert test="false()">Should not be invoked</sch:assert>
    </sch:rule>
  </sch:pattern>

  <sch:pattern name="MultiLineTime" is-a="Time">
    <sch:param formal="time" actual="time"/>
    <sch:param formal="hour" actual="hour"/>
    <sch:param formal="minute" actual="minute"/>
    <sch:param formal="second" actual="second"/>
  </sch:pattern>

  <sch:pattern name="Dummy pattern">
    <sch:rule context="dummy">
      <sch:assert test="false()">Should not be invoked</sch:assert>
    </sch:rule>
  </sch:pattern>
</sch:schema>
'''

STRON6 = '''\
<?xml version="1.0" encoding="UTF-8"?>
<sch:schema xmlns:sch="http://purl.oclc.org/dsdl/schematron">
  <sch:pattern name="Time">
    <sch:rule context="time">
      <sch:let name="hour" value="number(substring(.,1,2))"/>
      <sch:let name="minute" value="number(substring(.,4,2))"/>
      <sch:let name="second" value="number(substring(.,7,2))"/>

      <!-- CHECK FOR VALID HH:MM:SS -->
      <sch:assert test="string-length(.)=8 and substring(.,3,1)=':' and substring(.,6,1)=':'">The time element should contain a time in the format HH:MM:SS.</sch:assert>
      <sch:assert test="$hour>=0 and $hour&lt;=23">The hour must be a value between 0 and 23.</sch:assert>
      <sch:assert test="$minute>=0 and $minute&lt;=59">The minutes must be a value between 0 and 59.</sch:assert>
      <sch:assert test="$second>=0 and $second&lt;=59">The second must be a value between 0 and 59.</sch:assert>
    </sch:rule>
  </sch:pattern>
</sch:schema>
'''


INSTANCE1 = u'''<Person Title="Mr">
    <Name>Eddie</Name>
    <Gender>Male</Gender>
</Person>'''.encode(ENCODING)

INSTANCE2 = u'''\
<ex:Person Title="Mr" xmlns:ex="http://www.topologi.com/example">
    <ex:Name>Eddie</ex:Name>
    <ex:Gender>Male</ex:Gender>
</ex:Person>'''.encode(ENCODING)

INSTANCE3 = u'''\
<dog><ear/><bone/><ear/></dog>'''.encode(ENCODING)

INSTANCE4 = u'''\
<dog><ear/><bone/></dog>'''.encode(ENCODING)

INSTANCE5 = u'''\
<time>21:45:12</time>'''.encode(ENCODING)

INSTANCE6 = u'''\
<time>
    <hour>21</hour>
    <minute>45</minute>
    <second>12</second>
</time>'''.encode(ENCODING)


def normalize_text(text):
    #normalize text, a bit slow for large files with much non-ws
    return reduce(lambda a, b: (a and a[-1].isspace() and b.isspace()) and a or a+b, text, '')


class TestBasics(unittest.TestCase):
    def setUp(self):
        #self.isrc_factory = InputSource.DefaultFactory
        return

    def testStron1_1(self):
        output = run_stron(STRON1, INSTANCE1)
        #print output
        #normalize text, a bit slow for large files with much non-ws
        output = normalize_text(output)
        #print output

        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Processing pattern: Check structure\n'
        self.assertEqual(output, expected)
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testStron2_2(self):
        output = run_stron(STRON2, INSTANCE2)
        #print output
        #normalize text, a bit slow for large files with much non-ws
        output = normalize_text(output)
        #print output

        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Processing pattern: Check structure\nProcessing pattern: Check co-occurrence constraints\n'
        self.assertEqual(output, expected)
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testStron3_5(self):
        output = run_stron(STRON3, INSTANCE5)
        #print output
        #normalize text, a bit slow for large files with much non-ws
        output = normalize_text(output)
        #print output

        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Processing pattern: SingleLineTime\n'
        self.assertEqual(output, expected)
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testStron3_6(self):
        output = run_stron(STRON3, INSTANCE6)
        #print output
        #normalize text, a bit slow for large files with much non-ws
        output = normalize_text(output)
        #print output

        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Processing pattern: SingleLineTime\nAssertion failure:\nThe hour must be a value between 0 and 23.\nAssertion failure:\nThe minutes must be a value between 0 and 59.\n'
        self.assertEqual(output, expected)
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testStron4_5(self):
        output = run_stron(STRON4, INSTANCE5)
        #print output
        #normalize text, a bit slow for large files with much non-ws
        output = normalize_text(output)
        #print output

        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Processing pattern: MultiLineTime\nAssertion failure:\nThe hour must be a value between 0 and 23.\nAssertion failure:\nThe minutes must be a value between 0 and 59.\nAssertion failure:\nThe seconds must be a value between 0 and 23.\n'
        self.assertEqual(output, expected)
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testStron4_6(self):
        output = run_stron(STRON4, INSTANCE6)
        #print output
        #normalize text, a bit slow for large files with much non-ws
        output = normalize_text(output)
        #print output

        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Processing pattern: MultiLineTime\n'
        self.assertEqual(output, expected)
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testStron5_5(self):
        output = run_stron(STRON5, INSTANCE5)
        #print output
        #normalize text, a bit slow for large files with much non-ws
        output = normalize_text(output)
        #print output
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Processing pattern: Dummy pattern\nProcessing pattern: MultiLineTime\nAssertion failure:\nThe hour must be a value between 0 and 23.\nAssertion failure:\nThe minutes must be a value between 0 and 59.\nAssertion failure:\nThe seconds must be a value between 0 and 23.\nProcessing pattern: Dummy pattern\n'
        self.assertEqual(output, expected)
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testStron5_6(self):
        output = run_stron(STRON5, INSTANCE6)
        #print output
        #normalize text, a bit slow for large files with much non-ws
        output = normalize_text(output)
        #print output
        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Processing pattern: Dummy pattern\nProcessing pattern: MultiLineTime\nProcessing pattern: Dummy pattern\n'
        self.assertEqual(output, expected)
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testStron6_5(self):
        output = run_stron(STRON6, INSTANCE5)
        #print output
        #normalize text, a bit slow for large files with much non-ws
        output = normalize_text(output)
        #print output

        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Processing pattern: Time\n'
        self.assertEqual(output, expected)
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return

    def testStron6_6(self):
        output = run_stron(STRON6, INSTANCE6)
        #print output
        #normalize text, a bit slow for large files with much non-ws
        output = normalize_text(output)
        #print output

        expected = '<?xml version="1.0" encoding="UTF-8"?>\nProcessing schema: Processing pattern: Time\nAssertion failure:\nThe time element should contain a time in the format HH:MM:SS.\nAssertion failure:\nThe hour must be a value between 0 and 23.\nAssertion failure:\nThe minutes must be a value between 0 and 59.\n'
        self.assertEqual(output, expected)
        #self.assertRaises(AttributeError, binding.xbel.folder.)
        return


if __name__ == '__main__':
    unittest.main()




#The identity transform: duplicates the input to output
TRANSFORM = """<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

<xsl:template match="@*|node()">
  <xsl:copy>
    <xsl:apply-templates select="@*|node()"/>
  </xsl:copy>
</xsl:template>

</xsl:stylesheet>"""

SOURCE = """<spam id="eggs">I don't like spam</spam>"""

import amara
from Ft.Xml.Xslt import Processor
from Ft.Xml.InputSource import DefaultFactory
processor = Processor.Processor()
from Ft.Xml.Domlette import NonvalidatingReader
#Create a binding for the source document
doc = amara.parse(SOURCE)#, "http://spam.com/doc.xml")
src=DefaultFactory.fromString(TRANSFORM, "http://spam.com/identity.xslt")
processor.appendStylesheet(src)
result = processor.runNode(doc, "http://spam.com/doc.xml")
print result

